/*
 * Project: Sudoku Explainer
 * Copyright (C) 2006-2007 Nicolas Juillerat
 * Copyright (C) 2013-2020 Keith Corlett
 * Available under the terms of the Lesser General Public License (LGPL)
 */
package diuf.sudoku.solver.checks;

import diuf.sudoku.Grid;
import diuf.sudoku.Grid.ARegion;
import diuf.sudoku.Grid.Cell;
import org.junit.Test;
import static org.junit.Assert.*;
import static diuf.sudoku.TestHelp.*;
import diuf.sudoku.solver.AHint;

/**
 * Tests the NoHomelessValues class, an AWarningHinter.
 * @author Keith Corlett 2018 Mar
 */
public final class NoHomelessValuesTest {
	
private static final boolean REPRODUCE = REPRODUCE_ALL_TESTS;

private static final String PUZZLE =
"....41...9..5...81.7...96.....4.....3..6..14..8.1....3......83579.........2....1.";
//1465#C:\Users\User\Documents\SodukuPuzzles\MagicTour\top1465.d3.mt

//private static final String HOMELESS =
//"....41...9..5...81.7...96.....4.....3..6..14..8.1....3......83579.........2....1." + NL
//+"2568,2356,3568,2378,,,23579,2579,279,,2346,346,,237,237,2347,,,12458,,13458,238,238,,,25,24,1256,1256,15679,,235789,23578,2579,25679,26789,,25,579,,25789,2578,,,2789,2456,,45679,,2579,257,2579,25679,,146,146,146,279,12679,2467,,,,,,134568,238,123568,234568,24,26,246,4568,3456,,3789,356789,345678,479,,4679";
//// modified 1465#C:\Users\User\Documents\SodukuPuzzles\MagicTour\top1465.d3.mt
//// there are no 6's in box 2

/** Test of NoHomelessValues.getHints */
@Test
public void testGetHints() {
	final String testName = testName(this);
	NoHomelessValues hinter = new NoHomelessValues();
	// first a valid grid
	getHints(PUZZLE, hinter);
	AHint hint = accu.getHint();
	assertEquals(null, hint); // ie no hint

	// then an invalid grid, with a homeless value
	HINTS.clear(); // for when get hints is called repeatedly
	grid = new Grid(PUZZLE);
	// Splat the grid so that "The value 6 has no place in box 2"
	ARegion box2 = grid.boxs[1];
	for ( Cell c : box2.cells )
		c.canNotBe(6); // remove the maybe and update the indexes
	if ( !hinter.findHints(grid, accu) )
		fail(""+hinter+" found nothing.");
	hint = accu.getHint();
	assertEquals("The value 6 has no place in box 2", hint.toString());
	//System.out.println(hint);
	//System.out.println(hint.toHtml());
	assertHintEqualsStoredHintsHtml(this, hint, testName+".html", REPRODUCE);
}

}
