/*
 * Project: Sudoku Explainer
 * Copyright (C) 2006-2007 Nicolas Juillerat
 * Copyright (C) 2013-2020 Keith Corlett
 * Available under the terms of the Lesser General Public License (LGPL)
 */
package diuf.sudoku.solver.checks;

import diuf.sudoku.Grid;
import diuf.sudoku.Grid.Cell;
import org.junit.Test;
import static org.junit.Assert.*;
import static diuf.sudoku.TestHelp.*;
import diuf.sudoku.solver.AHint;
import diuf.sudoku.solver.UnsolvableException;


/**
 * Tests the NoMissingMaybes, an AWarningHinter.
 * @author Keith Corlett 2018 Mar
 */
public final class NoMissingMaybesTest {

private static final String PUZZLE =
"....41...9..5...81.7...96.....4.....3..6..14..8.1....3......83579.........2....1.";
//1465#C:\Users\User\Documents\SodukuPuzzles\MagicTour\top1465.d3.mt

private static final String MISSING_MAYBES
="....41...9..5...81.7...96.....4.....3..6..14..8.1....3......83579.........2....1." + NL
+",2356,3568,2378,,,23579,2579,279,,2346,346,,2367,2367,2347,,,12458,,13458,238,238,,,25,24,1256,1256,15679,,235789,23578,2579,25679,26789,,25,579,,25789,2578,,,2789,2456,,45679,,2579,257,2579,25679,,146,146,146,279,12679,2467,,,,,,134568,238,123568,234568,24,26,246,4568,3456,,3789,356789,345678,479,,4679";
//1465#C:\Users\User\Documents\SodukuPuzzles\MagicTour\top1465.d3.mt
//E2 no maybes remain.

/** Test of NoMissingMaybes.getHints. */
@Test
public void testNoMissingMaybes() {
	final String testName = testName(this);
	NoMissingMaybes validator = new NoMissingMaybes();
	getHints(PUZZLE, validator);
	AHint hint = accu.getHint();
	assertEquals(null, hint);

	grid = new Grid(PUZZLE); // constructor automagically fixes missing maybes
	Cell cell = grid.get("A1"); // A1 is an empty cell
	try {
		// clear all A1 maybes + indexes, not autosolving
		cell.canNotBeBits(cell.maybes.bits, null);
	} catch (UnsolvableException expected) {
		// Do nothing
	}
	if ( !validator.findHints(grid, accu) )
		fail(""+validator+" returned false.");
	hint = accu.getHint();
	if ( hint == null )
		fail(""+validator+" null hint");
	assert hint != null; // just for the ____in IDE
	if ( !(hint instanceof WarningHint) )
		fail(""+validator+" is not a WarningHint: "
				+hint.getClass().getSimpleName()+ ": "+hint.toString());
	//System.out.println(hint.toString());
	assertEquals("A1 has no potential values remaining", hint.toString());
	assertEquals("A1 has no potential values remaining", grid.invalidity);
}

}
