/*
 * Project: Sudoku Explainer
 * Copyright (C) 2006-2007 Nicolas Juillerat
 * Copyright (C) 2013-2020 Keith Corlett
 * Available under the terms of the Lesser General Public License (LGPL)
 */
package diuf.sudoku.solver.checks;

import diuf.sudoku.solver.AHint;
import org.junit.Test;
import static org.junit.Assert.*;
import static diuf.sudoku.TestHelp.*;

/**
 * Tests the TooFewClues class, an AWarningHinter.
 * @author Keith Corlett 2018 Mar
 */
public final class TooFewCluesTest {

private static final String PUZZLE =
"....41...9..5...81.7...96.....4.....3..6..14..8.1....3......83579.........2....1.";
//1465#C:\Users\User\Documents\SodukuPuzzles\MagicTour\top1465.d3.mt

private static final String TOO_FEW_CLUES =
"....41...9..5...81.7...96.....4...........14..8.1...........8...................." + NL
+"23568,2356,23568,23678,,,23579,23579,2379,,2346,2346,,2367,2367,2347,,,123458,,123458,238,238,,,235,234,12356,12356,1235679,,2356789,235678,23579,235679,236789,2356,2356,235679,236789,2356789,235678,,,236789,23456,,2345679,,235679,23567,23579,235679,23679,12346,12346,12346,23679,123679,23467,,23679,,,,1234568,2368,123568,234568,234,236,2346,234568,23456,234568,236789,2356789,2345678,23479,,234679";
//1465#C:\Users\User\Documents\SodukuPuzzles\MagicTour\top1465.d3.mt

/** Test of TooFewValues.getHints. */
@Test
public void testGetHints() {
	final String testName = testName(this);
	TooFewClues hinter = new TooFewClues();
	
	// first a valid grid --> no hint
	getHints(PUZZLE, hinter);
	AHint hint = accu.getHint();
	assertEquals(null, hint);
	
	char[] chars = TOO_FEW_CLUES.toCharArray();
	int cnt = 0;
	for ( int i=0; i<81; ++i )
		if ( chars[i] != '.' )
			++cnt;
	assert cnt == 15;

	// than a busted grid --> hint
	getHints(TOO_FEW_CLUES, hinter);
	hint = accu.getHint();
	if ( !(hint instanceof WarningHint) )
		fail("It's not a WarningHint it's a: "+hint.getClass().getSimpleName());
	assertEquals("Number of clues 15 is less than 17", hint.toString());
}

}
