/*
 * Project: Sudoku Explainer
 * Copyright (C) 2006-2007 Nicolas Juillerat
 * Copyright (C) 2013-2020 Keith Corlett
 * Available under the terms of the Lesser General Public License (LGPL)
 */
package diuf.sudoku.solver.hinters.align;

import static diuf.sudoku.TestHelp.*;
import static org.junit.Assert.*;
import diuf.sudoku.solver.AHint;
import org.junit.Test;


/**
 * Test the Aligned6Exclusion hinter.
 * @author Keith Corlett
 */
public final class Aligned6Exclusion_1CTest {

private static final boolean REPRODUCE = REPRODUCE_ALL_TESTS || false;

private static final Aligned6Exclusion_1C HINTER = new Aligned6Exclusion_1C(null);

private static final String PUZZLE
="..7..8.6.1...5...8...6.....7...6..35..4..7....9.2......5.4...2.47..2.3......16.4." + NL
+"2359,234,,139,349,,12459,,1234,,2346,2369,379,,2349,2479,79,,2389,2348,23589,,3479,12349,12479,157,12347,,128,28,189,,149,12489,,,23568,12368,,13589,389,,12689,189,1269,368,,3568,,348,1345,14678,178,1467,3689,,1689,,3789,39,16789,,1679,,,1689,89,,59,,158,169,2389,238,2389,35789,,,5789,,79";

/*
top1465.d5.2019-11-28.23-53-41.keep.log
top1465.d5.2019-12-12.10-16-50.A-6_1E.log
4#C:\Users\User\Documents\NetBeansProjects\DiufSudoku\top1465.d5.mt	..7..8.6.1...5...8...6.....7...6..35..4..7....9.2......5.4...2..7....3......16.4.
..7..8.6.1...5...8...6.....7...6..35..4..7....9.2......5.4...2.47..2.3......16.4.
2359,234,,139,349,,12459,,1234,,2346,2369,379,,2349,2479,79,,2389,2348,23589,,3479,12349,12479,157,12347,,128,28,189,,149,12489,,,23568,12368,,13589,389,,12689,189,1269,368,,3568,,348,1345,14678,178,1467,3689,,1689,,3789,39,16789,,1679,,,1689,89,,59,,158,169,2389,238,2389,35789,,,5789,,79
   11	    298,819,366	25	 208	  1	Unary Chain                   	Unary XY Chain: D5-8 (D5-8)
   11	    559,006,579	25	 208	  1	Aligned Hex                   	Aligned Hex: D1, D2, D4, D5, D8, F8 (D5-8)
*/

/** Test getHints in Aligned6Exclusion_1C, the "correct" version. */
@Test
public void test() {
	final String testName = testName(this);
	HINTER.setIsEnabled(true);
	getHints(PUZZLE, HINTER);
	AHint hint = find(HINTS, "Aligned Hex: D1, D2, D4, D5, D8, F8");
////KEEP THESE: For use in creating test cases.
//System.out.format("selectedCells : %s\n", diuf.sudoku.utils.Frmt.csv(hint.getAquaCells(0)));
//System.out.format("oranges       : %s\n", hint.getOranges(0));
//System.out.format("greens        : %s\n", hint.getGreens(0));
//System.out.format("reds          : %s\n", hint.getReds(0));
//System.out.format("redPots       : %s\n", hint.redPots);
//System.out.format("clueHtml      : %s\n", hint.getClueHtml(true));
	checkAquaCells(hint, "D1, D2, D4, D5, D8, F8");
	assertEquals(newPots("D1-139, D2-379, D4-189, D5-1359, D8-89, F8-59"), hint.getOranges(0));
	assertEquals(newPots("D9-35789"), hint.getGreens(0));
	assertEquals(newPots("D5-8"), hint.getReds(0));
	assertEquals(newPots("D5-8"), hint.redPots);
	checkClue(hint, "Look for an Aligned Hex at D1, D2, D4, D5, D8, F8");
	assertHintEqualsStoredHintsHtml(HINTER, hint, testName+".html", REPRODUCE||false);
}

}
