/*
 * Project: Sudoku Explainer
 * Copyright (C) 2006-2007 Nicolas Juillerat
 * Copyright (C) 2013-2020 Keith Corlett
 * Available under the terms of the Lesser General Public License (LGPL)
 */
package diuf.sudoku.solver.hinters.align;

import static diuf.sudoku.TestHelp.*;
import static org.junit.Assert.*;
import diuf.sudoku.solver.AHint;
import org.junit.Test;


/**
 * Test the Aligned6Exclusion hinter.
 * @author Keith Corlett
 */
public final class Aligned6Exclusion_2HTest {

private static final boolean REPRODUCE = REPRODUCE_ALL_TESTS || false;

private static final Aligned6Exclusion_2H HINTER = new Aligned6Exclusion_2H(null);

private static final String PUZZLE
="..7..8.6.16..5..98...6.....7...6..35..4.37...39.2......5.4.3.2.47..2.3......16.4."+NL
+"259,234,,13,49,,25,,1234,,,23,37,,24,47,,,89,348,359,,479,12,12,57,34,,128,28,18,,49,49,,,56,18,,59,,,269,18,26,,,56,,48,15,678,178,467,689,,1689,,78,,1678,,1679,,,16,89,,59,,58,16,28,238,2389,57,,,578,,79";

/*
69/4#C:\Users\User\Documents\NetBeansProjects\DiufSudoku\top1465.d5.mt
..7..8.6.16..5..98...6.....7...6..35..4.37...39.2......5.4.3.2.47..2.3......16.4.
259,234,,13,49,,25,,1234,,,23,37,,24,47,,,89,348,359,,479,12,12,57,34,,128,28,18,,49,49,,,56,18,,59,,,269,18,26,,,56,,48,15,678,178,467,689,,1689,,78,,1678,,1679,,,16,89,,59,,58,16,28,238,2389,57,,,578,,79
Aligned Hex: A1, B1, G1, I1, F2, F3 (I1-2)
*/

/** Test of Aligned6Exclusion.getHints method. */
@Test
public void test() {
	final String testName = testName(this);
	HINTER.setIsEnabled(true);
	getHints(PUZZLE, HINTER);
	AHint hint = find(HINTS, "Aligned Hex: A1, B1, G1, I1, F2, F3");
////KEEP THESE: For use in creating test cases.
//System.out.format("selectedCells : %s\n", diuf.sudoku.utils.Frmt.csv(hint.getAquaCells(0)));
//System.out.format("oranges       : %s\n", hint.getOranges(0));
//System.out.format("greens        : %s\n", hint.getGreens(0));
//System.out.format("reds          : %s\n", hint.getReds(0));
//System.out.format("redPots       : %s\n", hint.redPots);
//System.out.format("clueHtml      : %s\n", hint.getClueHtml(true));
	checkAquaCells(hint, "A1, B1, G1, I1, F2, F3");
	assertEquals(newPots("A1-259, B1-234, F2-24, F3-12, G1-25, I1-134"), hint.getOranges(0));
	assertEquals(newPots("D1-13, E1-49"), hint.getGreens(0));
	assertEquals(newPots("I1-2"), hint.getReds(0));
	assertEquals(newPots("I1-2"), hint.redPots);
	checkClue(hint, "Look for an Aligned Hex at A1, B1, G1, I1, F2, F3");
	assertHintEqualsStoredHintsHtml(HINTER, hint, testName+".html", REPRODUCE);
}



//private static final String PUZZLE_1
//="....8.9.78......5.9..6.1.8.2.6..4.9..49...1...8.9..3464.8.5....6...7.8....23.8..."+NL
//+"13,26,134,245,,235,,126,,,26,134,247,349,2379,246,,134,,57,57,,234,,24,,234,,137,,1578,13,,57,,58,357,,,578,36,3567,,27,258,157,,157,,12,257,,,,,1379,,12,,26,27,12367,19,,1359,135,124,,29,,123,145,157,1579,,,1469,,4567,167,1459";
//
///*
//This hint is MIA in the latest version 2019-11-22 of Aligned6Exclusion
//from top1465.d5.2019-11-12.11-07-47.keep.log, which was still there 2019-11-17,
//which was the date of the last release.
//79#C:\Users\User\Documents\NetBeansProjects\DiufSudoku\top1465.d5.mt	....8...7.......5.9..6.1...2.6..4.9.......1.....9..3.64.8.5....6...7.8....23.....
//....8.9.78......5.9..6.1.8.2.6..4.9..49...1...8.9..3464.8.5....6...7.8....23.8...
//13,26,134,245,,235,,126,,,26,134,247,349,2379,246,,134,,57,57,,234,,24,,234,,137,,1578,13,,57,,58,357,,,578,36,3567,,27,258,157,,157,,12,257,,,,,1379,,12,,26,27,12367,19,,1359,135,124,,29,,123,145,157,1579,,,1469,,4567,167,1459
//   34	     23,102,442	30	 146	  1	Aligned Hex                   	Aligned Hex: D7, H7, I8, G9, H9, I9 (H7-1)
//   34	     28,262,646	31	 137	 10	Dynamic Plus                  	Cell Reduction XY Chain (0): H1 so D4+7 (D4+7 D4-158)
//*/
//
///** Test of Aligned6Exclusion.getHints method. */
//@Test
//public void test1() {
//	final String testName = testName(this);
//	HINTER.setIsEnabled(true);
//	getHints(PUZZLE_1, HINTER);
//	AHint hint = find(HINTS, "Aligned Hex: D7, H7, I8, G9, H9, I9");
//////KEEP THESE: For use in creating test cases.
//System.out.format("selectedCells : %s\n", diuf.sudoku.utils.Frmt.csv(hint.getAquaCells(0)));
//System.out.format("greens        : %s\n", hint.getGreens(0));
//System.out.format("reds          : %s\n", hint.getReds(0));
//System.out.format("redPots       : %s\n", hint.redPots);
//System.out.format("clueHtml      : %s\n", hint.getClueHtml(true));
//	checkAquaCells(hint, "D7, H7, I8, G9, H9, I9");
////	assertEquals(newPots("A1-259, B1-234, D1-13, E1-49, F2-24, F3-12, G1-25"), hint.getGreens(0));
////	assertEquals(newPots("A1-259, B1-234, F2-24, F3-12, G1-25, I1-2"), hint.getReds(0));
//	assertEquals(newPots("H7-1"), hint.redPots);
//	checkClue(hint, "Look for an Aligned Hex at D7, H7, I8, G9, H9, I9");
//	assertHintEqualsStoredHintsHtml(HINTER, hint, testName+".html", REPRODUCE||true);
//}

}
