/*
 * Project: Sudoku Explainer
 * Copyright (C) 2006-2007 Nicolas Juillerat
 * Copyright (C) 2013-2020 Keith Corlett
 * Available under the terms of the Lesser General Public License (LGPL)
 */
package diuf.sudoku.solver.hinters.align;

import static diuf.sudoku.TestHelp.*;
import static org.junit.Assert.*;
import diuf.sudoku.solver.AHint;
import org.junit.Test;


/**
 * Test the Aligned7Exclusion hinter.
 * @author Keith Corlett
 */
public final class Aligned7Exclusion_2HTest {

private static final boolean REPRODUCE = REPRODUCE_ALL_TESTS || false;

private static final Aligned7Exclusion_2H HINTER = new Aligned7Exclusion_2H(null);

private static final String PUZZLE
="7.....4...2..7..83..3..8..9...5.63...6..2..9...1..7..6...3.29...3..4..6...9..1.35"+NL
+",189,568,1269,1369,359,,125,12,19,,56,149,,45,156,,,146,145,,126,156,,1267,127,,289,79,247,,189,,,1247,12478,35,,457,18,,34,178,,1478,23,89,,489,39,,25,245,,168,1578,678,,568,,,147,1478,1258,,257,789,,59,1278,,1278,2468,478,,678,68,,278,,";

/*
1#C:\Users\User\Documents\NetBeansProjects\DiufSudoku\top1465.d5.mt
7.....4...2..7..83..3..8..9...5.63...6..2..9...1..7..6...3.29...3..4..6...9..1.35
,189,568,1269,1369,359,,125,12,19,,56,149,,45,156,,,146,145,,126,156,,1267,127,,289,79,247,,189,,,1247,12478,35,,457,18,,34,178,,1478,23,89,,489,39,,25,245,,168,1578,678,,568,,,147,1478,1258,,257,789,,59,1278,,1278,2468,478,,678,68,,278,,
   39	     42,110,289	28	 157	  1	Aligned Sept                  	Aligned Sept: C1, A2, C2, A3, B4, B6, B9 (A3-1)
*/

/** Test of Aligned7Exclusion.getHints method. */
@Test
public void test() {
	final String testName = testName(this);
	HINTER.setIsEnabled(true);
	getHints(PUZZLE, HINTER);
	AHint hint = find(HINTS, "Aligned Sept: C1, A2, C2, A3, B4, B6, B9");
////KEEP THESE: For use in creating test cases.
//System.out.format("selectedCells : %s\n", diuf.sudoku.utils.Frmt.csv(hint.getAquaCells(0)));
//System.out.format("oranges       : %s\n", hint.getOranges(0));
//System.out.format("greens        : %s\n", hint.getGreens(0));
//System.out.format("reds          : %s\n", hint.getReds(0));
//System.out.format("redPots       : %s\n", hint.redPots);
//System.out.format("clueHtml      : %s\n", hint.getClueHtml(true));
	checkAquaCells(hint, "C1, A2, C2, A3, B4, B6, B9");
	assertEquals(newPots("A2-19, A3-46, B4-79, B6-89, B9-478, C1-568, C2-56"), hint.getOranges(0));
	assertEquals(newPots("B1-189, B3-145"), hint.getGreens(0));
	assertEquals(newPots("A3-1"), hint.getReds(0));
	assertEquals(newPots("A3-1"), hint.redPots);
	checkClue(hint, "Look for an Aligned Sept at C1, A2, C2, A3, B4, B6, B9");
	assertHintEqualsStoredHintsHtml(HINTER, hint, testName+".html", REPRODUCE);
}

}
