/*
 * Project: Sudoku Explainer
 * Copyright (C) 2006-2007 Nicolas Juillerat
 * Copyright (C) 2013-2020 Keith Corlett
 * Available under the terms of the Lesser General Public License (LGPL)
 */
package diuf.sudoku.solver.hinters.align2;

import diuf.sudoku.Settings;
import diuf.sudoku.Tech;
import static diuf.sudoku.TestHelp.*;
import static org.junit.Assert.*;
import diuf.sudoku.solver.AHint;
import org.junit.Test;


/**
 * Test the Aligned10Exclusion hinter.
 * @author Keith Corlett
 */
public final class Aligned10Exclusion_2HTest {

private static final boolean REPRODUCE = REPRODUCE_ALL_TESTS || false;

static {
	// the AlignedExclusion constructor reads this setting
	Settings.THE.set("isa10ehacked", true);
	Settings.THE.save();
}
private static final AlignedExclusion HINTER = new AlignedExclusion(Tech.AlignedDec);

private static final String PUZZLE
=".4.3..9..973..5.2.8........6.9...352..5....9...4...6.....9...683...42.79.9.16...." + NL
+"125,,126,,278,1678,,18,567,,,,46,18,,48,,146,,1256,126,2467,1279,14679,457,34,3567,,18,,478,178,1478,,,,127,1238,,268,1238,1368,478,,147,127,1238,,258,2359,1389,,18,17,124,125,127,,357,37,1245,,,,56,168,58,,,15,,,245,,78,,,78,245,34,35";

/*
13#C:\Users\User\Documents\NetBeansProjects\DiufSudoku\top1465.d5.mt
.4.3..9..973..5.2.8........6.9...352..5....9...4...6.....9...683...42.79.9.16....
125,,126,,278,1678,,18,567,,,,46,18,,48,,146,,1256,126,2467,1279,14679,457,34,3567,,18,,478,178,1478,,,,127,1238,,268,1238,1368,478,,147,127,1238,,258,2359,1389,,18,17,124,125,127,,357,37,1245,,,,56,168,58,,,15,,,245,,78,,,78,245,34,35
   35	    303,749,664	29	 154	  1	Aligned Dec                   	Aligned Dec: E1, E2, E3, D4, F4, D5, E5, F5, D6, F6 (F6-3)
*/

/** Test of Aligned10Exclusion.getHints method. */
@Test
public void test() {
	final String testName = testName(this);
	HINTER.setIsEnabled(true);
	getHints(PUZZLE, HINTER, true);
	AHint hint = find(HINTS, "Aligned Dec: E1, E2, E3, D4, F4, D5, E5, F5, D6, F6");
////KEEP THESE: For use in (re)creating test case(s).
//System.out.format("selectedCells : %s\n", diuf.sudoku.utils.Frmt.csv(hint.getAquaCells(0)));
//System.out.format("oranges       : %s\n", hint.getOranges(0));
//System.out.format("greens        : %s\n", hint.getGreens(0));
//System.out.format("reds          : %s\n", hint.getReds(0));
//System.out.format("redPots       : %s\n", hint.redPots);
//System.out.format("clueHtml      : %s\n", hint.getClueHtml(true));
	checkAquaCells(hint, "E1, E2, E3, D4, F4, D5, E5, F5, D6, F6");
	assertEquals(newPots("D4-478, D5-268, D6-258, E1-278, E2-18, E3-1279, E5-1238, F4-1478, F5-1368, F6-189"), hint.getOranges(0));
	assertEquals(newPots("E4-178, E6-2359"), hint.getGreens(0));
	assertEquals(newPots("F6-3"), hint.getReds(0));
	assertEquals(newPots("F6-3"), hint.redPots);
	checkClue(hint, "Look for an Aligned Dec at E1, E2, E3, D4, F4, D5, E5, F5, D6, F6");
	assertHintEqualsStoredHintsHtml(HINTER, hint, testName+".html", REPRODUCE);
}

}
