/*
 * Project: Sudoku Explainer
 * Copyright (C) 2006-2007 Nicolas Juillerat
 * Copyright (C) 2013-2020 Keith Corlett
 * Available under the terms of the Lesser General Public License (LGPL)
 */
package diuf.sudoku.solver.hinters.align2;

import diuf.sudoku.Tech;
import static diuf.sudoku.TestHelp.*;
import static org.junit.Assert.*;

import diuf.sudoku.solver.AHint;
import org.junit.Test;
import diuf.sudoku.solver.hinters.AHinter;


/**
 * Test the Aligned2Exclusion hinter.
 * @author Keith Corlett
 */
public final class Aligned2ExclusionTest {

private static final boolean REPRODUCE = REPRODUCE_ALL_TESTS || false;

private static final AlignedExclusion HINTER = new AlignedExclusion(Tech.AlignedPair);


private static final String PUZZLE1
="....4.....147....6..29.3...2...7.19..9....56.6.5.91..35.....72...6.....1.....763." + NL
+"3789,56,3789,1568,,568,239,1578,29,39,,,,258,258,39,58,,78,56,,,1568,,48,14578,4578,,348,38,56,,56,,,48,1478,,178,2348,238,248,,,2478,,478,,248,,,248,478,,,348,1389,13468,1368,4689,,,489,34789,23478,,23458,2358,24589,489,458,,1489,248,189,12458,1258,,,,4589";
/*
....4.....147....6..29.3...2...7.19..9....56.6.5.91..35.....72...6.....1.....763.
3789,56,3789,1568,,568,239,1578,29,39,,,,258,258,39,58,,78,56,,,1568,,48,14578,4578,,348,38,56,,56,,,48,1478,,178,2348,238,248,,,2478,,478,,248,,,248,478,,,348,1389,13468,1368,4689,,,489,34789,23478,,23458,2358,24589,489,458,,1489,248,189,12458,1258,,,,4589
744#C:\Users\User\Documents\NetBeansProjects\DiufSudoku\top1465.d4.mt
Aligned Pair: H3, I3 (H3-8)
*/

/** Test of getHints method, of class AlignedPairExclusionV2.
 * This is not a sure test, apparently. */
@Test
public void test1() {
	final String testName = testName(this);
	getHints(PUZZLE1, HINTER, true);

	AHint hint = find(HINTS, "Aligned Pair: H3, I3");
////KEEP THESE: For use in (re)creating test case(s).
//System.out.format("selectedCells : %s\n", diuf.sudoku.utils.Frmt.csv(hint.getAquaCells(0)));
//System.out.format("oranges       : %s\n", hint.getOranges(0));
//System.out.format("greens        : %s\n", hint.getGreens(0));
//System.out.format("reds          : %s\n", hint.getReds(0));
//System.out.format("redPots       : %s\n", hint.redPots);
//System.out.format("clueHtml      : %s\n", hint.getClueHtml(true));
	checkAquaCells(hint, "H3, I3");
	assertEquals(newPots("H3-1457, I3-4578"), hint.getOranges(0));
	assertEquals(newPots("A3-78, G3-48, H2-58"), hint.getGreens(0));
	assertEquals(newPots("H3-8"), hint.getReds(0));
	assertEquals(newPots("H3-8"), hint.redPots);
	checkClue(hint, "Look for an Aligned Pair at H3, I3");
	assertHintEqualsStoredHintsHtml(HINTER, hint, testName+".html", REPRODUCE);
}


private static final String PUZZLE2
="..7..8.6.1...5...8...6.....7...6..35..4..7....9.2......5.4.3.2.47..2.3......16.4."+NL
+"259,234,,13,349,,2459,,1234,,2346,2369,37,,249,2479,79,,2389,2348,23589,,3479,124,12479,57,1234,,128,28,189,,149,12489,,,23568,12368,,359,38,,12689,189,1269,368,,3568,,348,145,14678,178,1467,689,,1689,,789,,16789,,1679,,,1689,89,,59,,158,169,2389,238,2389,578,,,5789,,79";
/*
Using: AlignedPairExclusion20190918_old (that's a class name)
23/4#C:/Users/User/Documents/NetBeansProjects/DiufSudoku/top1465.d5.mt
..7..8.6.1...5...8...6.....7...6..35..4..7....9.2......5.4.3.2.47..2.3......16.4.
259,234,,13,349,,2459,,1234,,2346,2369,37,,249,2479,79,,2389,2348,23589,,3479,124,12479,57,1234,,128,28,189,,149,12489,,,23568,12368,,359,38,,12689,189,1269,368,,3568,,348,145,14678,178,1467,689,,1689,,789,,16789,,1679,,,1689,89,,59,,158,169,2389,238,2389,578,,,5789,,79
Aligned Pair: I8, D9 (I8-9)
C:/Users/User/Documents/SodukuPuzzles/Test/top1465.d5.4.23.AlignedPairExclusion20190918.txt
*/

/** Test of getHint method, of class AlignedPairExclusionV2.
 * This is a better test, apparently. */
@Test
public void test2() {
	final String testName = testName(this);
	getHints(PUZZLE2, HINTER, true);

	AHint hint = find(HINTS, "Aligned Pair: I8, D9"); // (I8-9)
	try {
		checkAquaCells(hint, "I8, D9");
		assertEquals(newPots("D9-578, I8-16"), hint.getOranges(0));
		// assert-equals the greenPots manually because JUnit is a piece of s__t.
		String expectedGreens = "D8-89, F8-59, I9-79";
		String actualGreens = hint.getGreens(0).toString();
		if ( !expectedGreens.equals(actualGreens) )
			fail("expected: "+expectedGreens+NL+" but was: "+actualGreens);
		assertEquals(newPots("I8-9"), hint.getReds(0));
		assertEquals(newPots("I8-9"), hint.redPots);
		checkClue(hint, "Look for an Aligned Pair at I8, D9");
		assertHintEqualsStoredHintsHtml(HINTER, hint, testName+".html", REPRODUCE||false);
	} catch (Throwable t) {
		if ( hint == null ) {
			System.out.format("hint is null\n");
		} else {
			//KEEP THESE: For use in (re)creating test case(s).
			System.out.format("selectedCells : %s\n", diuf.sudoku.utils.Frmt.csv(hint.getAquaCells(0)));
			System.out.format("oranges       : %s\n", hint.getOranges(0));
			System.out.format("greens        : %s\n", hint.getGreens(0));
			System.out.format("reds          : %s\n", hint.getReds(0));
			System.out.format("redPots       : %s\n", hint.redPots);
			System.out.format("clueHtml      : %s\n", hint.getClueHtml(true));
		}
	}
}

}
