/*
 * Project: Sudoku Explainer
 * Copyright (C) 2006-2007 Nicolas Juillerat
 * Copyright (C) 2013-2020 Keith Corlett
 * Available under the terms of the Lesser General Public License (LGPL)
 */
package diuf.sudoku.solver.hinters.align2;

import diuf.sudoku.Tech;
import org.junit.Test;
import static diuf.sudoku.TestHelp.*;
import diuf.sudoku.solver.AHint;
import static org.junit.Assert.*;


/**
 * Tests the Aligned3Exclusion hinter.
 * @author Keith Corlett 2018 Apr
 */
public final class Aligned3ExclusionTest {
	
private static final boolean REPRODUCE = REPRODUCE_ALL_TESTS || false;

private static final AlignedExclusion HINTER = new AlignedExclusion(Tech.AlignedTriple);

private static final String PUZZLE
="7.....4...2..7..8.4.3..8..9...5.63...6..2..9...1..7..6...3.29...3..4..6..49..1.35" + NL
+",89,68,26,13,59,,25,13,19,,56,149,,34,56,,13,,15,,26,15,,267,27,,89,789,247,,189,,,14,2478,35,,457,18,,34,1578,,78,2359,589,,48,39,,258,245,,1568,1578,78,,568,,,147,478,158,,2578,789,,59,178,,278,268,,,78,68,,278,,";
// TEST_DATA_DIR+"top1465.d3.1.txt
/*
7.....4...2..7..8.4.3..8..9...5.63...6..2..9...1..7..6...3.29...3..4..6..49..1.35
,89,68,26,13,59,,25,13,19,,56,149,,34,56,,13,,15,,26,15,,267,27,,89,789,247,,189,,,14,2478,35,,457,18,,34,1578,,78,2359,589,,48,39,,258,245,,1568,1578,78,,568,,,147,478,158,,2578,789,,59,178,,278,268,,,78,68,,278,,
*/

/** Test of Aligned3Exclusion.getHints */
@Test
public void test() {
	final String testName = testName(this);
	HINTER.setIsEnabled(true);
	getHints(PUZZLE, HINTER, true);
//System.out.format("ateMaxNumCandidateCells = %,d\n", Aligned3Exclusion.ateMaxNumCandidateCells);

	AHint hint = null;
	int testNumber = 0;
	try {

	// HINT 1
	++testNumber;
	hint = find(HINTS, "Aligned Triple: A2, B4, B6");
	checkAquaCells(hint, "A2, B4, B6");
	assertEquals(newPots("A2-19, B4-79, B6-589"), hint.getOranges(0));
	assertEquals(newPots("B1-89, B3-15"), hint.getGreens(0));
	assertEquals(newPots("B4-8"), hint.getReds(0));
	assertEquals(newPots("B4-8"), hint.redPots);
	checkClue(hint, "Look for an Aligned Triple at A2, B4, B6");
	assertHintEqualsStoredHintsHtml(this, hint, testName+"_1.html", REPRODUCE, false);

	// HINT 2
	++testNumber;
	hint = find(HINTS, "Aligned Triple: B3, A4, A6"); // order changed
	checkAquaCells(hint, "B3, A6, A4");
	assertEquals(newPots("A4-89, A6-235, B3-15"), hint.getOranges(0));
	assertEquals(newPots("A2-19, B6-589"), hint.getGreens(0));
	assertEquals(newPots("A6-9"), hint.getReds(0));
	assertEquals(newPots("A6-9"), hint.redPots);
	checkClue(hint, "Look for an Aligned Triple at B3, A4, A6");
	assertHintEqualsStoredHintsHtml(this, hint, testName+"_2.html", REPRODUCE, false);

	// HINT 3
	++testNumber;
	hint = find(HINTS, "Aligned Triple: A2, B6, B7");
	checkAquaCells(hint, "A2, B7, B6");
	assertEquals(newPots("A2-19, B6-589, B7-157"), hint.getOranges(0));
	assertEquals(newPots("B1-89, B3-15"), hint.getGreens(0));
	assertEquals(newPots("B7-8"), hint.getReds(0));
	assertEquals(newPots("B7-8"), hint.redPots);
	checkClue(hint, "Look for an Aligned Triple at A2, B6, B7");
	assertHintEqualsStoredHintsHtml(this, hint, testName+"_3.html", REPRODUCE);
	
	} catch (Throwable t) {
		System.out.format("testNumber            : %s\n", testNumber);
		if ( hint == null ) {
			System.out.println("hint is null");
		} else {
			//KEEP THESE: For use in (re)creating test case(s).
			System.out.format("selectedCells : %s\n", diuf.sudoku.utils.Frmt.csv(hint.getAquaCells(0)));
			System.out.format("oranges       : %s\n", hint.getOranges(0));
			System.out.format("greens        : %s\n", hint.getGreens(0));
			System.out.format("reds          : %s\n", hint.getReds(0));
			System.out.format("redPots       : %s\n", hint.redPots);
			System.out.format("clueHtml      : %s\n", hint.getClueHtml(true));
		}
		throw t;
	}

	if ( REPRODUCE )
		fail("the run to produce test HTML's is not a real test");
}

}