/*
 * Project: Sudoku Explainer
 * Copyright (C) 2006-2007 Nicolas Juillerat
 * Copyright (C) 2013-2020 Keith Corlett
 * Available under the terms of the Lesser General Public License (LGPL)
 */
package diuf.sudoku.solver.hinters.align2;

import diuf.sudoku.Settings;
import diuf.sudoku.Tech;
import static diuf.sudoku.TestHelp.*;
import static org.junit.Assert.*;
import diuf.sudoku.solver.AHint;
import org.junit.Test;


/**
 * Test the Aligned4Exclusion hinter.
 * @author Keith Corlett
 */
public final class Aligned4ExclusionTest {

private static final boolean REPRODUCE = REPRODUCE_ALL_TESTS || false;

private static final String PUZZLE
="6.....3.7.5..9..8...2..6.598.5...7...7..5..4......1..51263.957..4..2..6...8..7..2" + NL
+",189,149,12458,148,2458,,12,,347,,1347,127,,23,26,,146,347,138,,478,137,,14,,,,1369,,2469,346,234,,1239,136,239,,139,2689,,238,12689,,1368,2349,369,349,6789,3678,,2689,239,,,,,,48,,,,48,3579,,379,158,,58,189,,138,359,39,,1456,146,,149,139,";

/*
top1465.d5.2019-11-28.23-53-41.keep.log
top1465.d5.2019-12-12.10-16-50.A-6_1E.log
15#C:\Users\User\Documents\NetBeansProjects\DiufSudoku\top1465.d5.mt	6.....3...5..9..8...2..6..98.....7...7..5..4......1..51..3..5...4..2..6...8..7..2
6.....3.7.5..9..8...2..6.598.5...7...7..5..4......1..51263.957..4..2..6...8..7..2
,189,149,12458,148,2458,,12,,347,,1347,127,,23,26,,146,347,138,,478,137,,14,,,,1369,,2469,346,234,,1239,136,239,,139,2689,,238,12689,,1368,2349,369,349,6789,3678,,2689,239,,,,,,48,,,,48,3579,,379,158,,58,189,,138,359,39,,1456,146,,149,139,
   23	     54,186,381	31	 158	  1	Unary Chain                   	Unary XY Chain: D1-2 (D1-2)
   23	     18,308,933	31	 158	  1	Aligned Quad                  	Aligned Quad: G3, G8, B9, H9 (G8-1)
*/

/** Test getHints in Aligned4Exclusion: the only version is the "correct" version. */
@Test
public void test() {
	final String testName = testName(this);
	// the constructor reads isa4ehacked;
	boolean preHacked = Settings.THE.set("isa4ehacked", false);
	AlignedExclusion hinter = new AlignedExclusion(Tech.AlignedQuad);
	Settings.THE.set("isa4ehacked", preHacked); // put it back
	Settings.THE.save();
	hinter.setIsEnabled(true);
	getHints(PUZZLE, hinter, true);
	AHint hint = find(HINTS, "Aligned Quad: G3, G8, B9, H9");
////KEEP THESE, it's how I created this test
//System.out.format("selectedCells : %s\n", diuf.sudoku.utils.Frmt.csv(hint.getAquaCells(0)));
//System.out.format("oranges       : %s\n", hint.getOranges(0));
//System.out.format("greens        : %s\n", hint.getGreens(0));
//System.out.format("reds          : %s\n", hint.getReds(0));
//System.out.format("redPots       : %s\n", hint.redPots);
//System.out.format("clueHtml      : %s\n", hint.getClueHtml(true));
	checkAquaCells(hint, "G3, G8, B9, H9");
	assertEquals(newPots("B9-39, G3-14, G8-89, H9-139"), hint.getOranges(0));
	assertEquals(newPots("G9-149"), hint.getGreens(0));
	assertEquals(newPots("G8-1"), hint.getReds(0));
	assertEquals(newPots("G8-1"), hint.redPots);
	checkClue(hint, "Look for an Aligned Quad at G3, G8, B9, H9");
	assertHintEqualsStoredHintsHtml(hinter, hint, testName+".html", REPRODUCE);
}

}
