/*
 * Project: Sudoku Explainer
 * Copyright (C) 2006-2007 Nicolas Juillerat
 * Copyright (C) 2013-2020 Keith Corlett
 * Available under the terms of the Lesser General Public License (LGPL)
 */
package diuf.sudoku.solver.hinters.align2;

import diuf.sudoku.Settings;
import diuf.sudoku.Tech;
import static diuf.sudoku.TestHelp.*;
import static org.junit.Assert.*;
import diuf.sudoku.solver.AHint;
import org.junit.Test;


/**
 * Test the AlignedPairExclusion hinter.
 * @author Keith Corlett
 */
public final class Aligned5Exclusion_1CTest {

private static final boolean REPRODUCE = REPRODUCE_ALL_TESTS || false;

static {
	// the AlignedExclusion constructor reads this setting
	Settings.THE.set("isa5ehacked", false);
	Settings.THE.save();
}
private static final AlignedExclusion HINTER = new AlignedExclusion(Tech.AlignedPent);

private static final String PUZZLE
="7.8...4...2..7..83..32.8..9...5.63...6..2..9...1..7..6...3.29...3..4..6...9..1.35" + NL
+",19,,69,369,359,,125,12,19,,56,149,,45,156,,,46,45,,,15,,167,17,,289,79,247,,189,,,1247,12478,35,,457,18,,34,178,,1478,23,89,,489,39,,25,245,,168,1578,67,,568,,,147,1478,1258,,257,789,,59,1278,,1278,2468,478,,678,68,,278,,";
// 1#C:\Users\User\Documents\NetBeansProjects\DiufSudoku\top1465.d5.mt

/*
top1465.d5.2019-11-28.23-53-41.keep.log
top1465.d5.2019-12-12.10-16-50.A-6_1E.log
1#C:\Users\User\Documents\NetBeansProjects\DiufSudoku\top1465.d5.mt	7.....4...2..7..8...3..8..9...5..3...6..2..9...1..7..6...3..9...3..4..6...9..1..5
7.8...4...2..7..83..32.8..9...5.63...6..2..9...1..7..6...3.29...3..4..6...9..1.35
,19,,69,369,359,,125,12,19,,56,149,,45,156,,,46,45,,,15,,167,17,,289,79,247,,189,,,1247,12478,35,,457,18,,34,178,,1478,23,89,,489,39,,25,245,,168,1578,67,,568,,,147,1478,1258,,257,789,,59,1278,,1278,2468,478,,678,68,,278,,
   48	     65,851,321	30	 141	  1	Unary Chain                   	Unary XY Chain: I4-2 (I4-2)
   48	     30,903,003	30	 141	  1	Aligned Pent                  	Aligned Pent: H1, H3, H4, G6, H7 (H1-1)
*/

/** Test getHints in Aligned5Exclusion_1C, the "correct" version. */
@Test
public void test() {
	final String testName = testName(this);
	HINTER.setIsEnabled(true);
	getHints(PUZZLE, HINTER, true);
	AHint hint = find(HINTS, "Aligned Pent: H1, H3, H4, G6, H7");
////KEEP THESE: For use in creating test cases.
//System.out.format("selectedCells : %s\n", diuf.sudoku.utils.Frmt.csv(hint.getAquaCells(0)));
//System.out.format("oranges       : %s\n", hint.getOranges(0));
//System.out.format("greens        : %s\n", hint.getGreens(0));
//System.out.format("reds          : %s\n", hint.getReds(0));
//System.out.format("redPots       : %s\n", hint.redPots);
//System.out.format("clueHtml      : %s\n", hint.getClueHtml(true));
	checkAquaCells(hint, "H1, H3, H4, G6, H7");
	assertEquals(newPots("G6-25, H1-25, H3-17, H4-1247, H7-147"), hint.getOranges(0));
	assertEquals(newPots("H6-245"), hint.getGreens(0));
	assertEquals(newPots("H1-1"), hint.getReds(0));
	assertEquals(newPots("H1-1"), hint.redPots);
	checkClue(hint, "Look for an Aligned Pent at H1, H3, H4, G6, H7");
	assertHintEqualsStoredHintsHtml(HINTER, hint, testName+".html", REPRODUCE||false);
}

}
