/*
 * Project: Sudoku Explainer
 * Copyright (C) 2006-2007 Nicolas Juillerat
 * Copyright (C) 2013-2020 Keith Corlett
 * Available under the terms of the Lesser General Public License (LGPL)
 */
package diuf.sudoku.solver.hinters.align2;

import diuf.sudoku.Settings;
import diuf.sudoku.Tech;
import static diuf.sudoku.TestHelp.*;
import static org.junit.Assert.*;
import diuf.sudoku.solver.AHint;
import org.junit.Test;


/**
 * Test the Aligned6Exclusion hinter.
 * @author Keith Corlett
 */
public final class Aligned6Exclusion_2HTest {

private static final boolean REPRODUCE = REPRODUCE_ALL_TESTS || false;

static {
	// the AlignedExclusion constructor reads this setting
	Settings.THE.set("isa6ehacked", true);
	Settings.THE.save();
}
private static final AlignedExclusion HINTER = new AlignedExclusion(Tech.AlignedHex);

private static final String PUZZLE
="..7..8.6.16..5..98...6.....7...6..35..4.37...39.2......5.4.3.2.47..2.3......16.4."+NL
+"259,234,,13,49,,25,,1234,,,23,37,,24,47,,,89,348,359,,479,12,12,57,34,,128,28,18,,49,49,,,56,18,,59,,,269,18,26,,,56,,48,15,678,178,467,689,,1689,,78,,1678,,1679,,,16,89,,59,,58,16,28,238,2389,57,,,578,,79";

/*
69/4#C:\Users\User\Documents\NetBeansProjects\DiufSudoku\top1465.d5.mt
..7..8.6.16..5..98...6.....7...6..35..4.37...39.2......5.4.3.2.47..2.3......16.4.
259,234,,13,49,,25,,1234,,,23,37,,24,47,,,89,348,359,,479,12,12,57,34,,128,28,18,,49,49,,,56,18,,59,,,269,18,26,,,56,,48,15,678,178,467,689,,1689,,78,,1678,,1679,,,16,89,,59,,58,16,28,238,2389,57,,,578,,79
Aligned Hex: A1, B1, G1, I1, F2, F3 (I1-2)
*/

/** Test of Aligned6Exclusion.getHints method. */
@Test
public void test() {
	final String testName = testName(this);
	HINTER.setIsEnabled(true);
	getHints(PUZZLE, HINTER, true);
	AHint hint = find(HINTS, "Aligned Hex: A1, B1, G1, I1, F2, F3");
////KEEP THESE: For use in creating test cases.
//System.out.format("selectedCells : %s\n", diuf.sudoku.utils.Frmt.csv(hint.getAquaCells(0)));
//System.out.format("oranges       : %s\n", hint.getOranges(0));
//System.out.format("greens        : %s\n", hint.getGreens(0));
//System.out.format("reds          : %s\n", hint.getReds(0));
//System.out.format("redPots       : %s\n", hint.redPots);
//System.out.format("clueHtml      : %s\n", hint.getClueHtml(true));
	checkAquaCells(hint, "A1, B1, G1, I1, F2, F3");
	assertEquals(newPots("A1-259, B1-234, F2-24, F3-12, G1-25, I1-134"), hint.getOranges(0));
	assertEquals(newPots("D1-13, E1-49"), hint.getGreens(0));
	assertEquals(newPots("I1-2"), hint.getReds(0));
	assertEquals(newPots("I1-2"), hint.redPots);
	checkClue(hint, "Look for an Aligned Hex at A1, B1, G1, I1, F2, F3");
	assertHintEqualsStoredHintsHtml(HINTER, hint, testName+".html", REPRODUCE);
}

}
