/*
 * Project: Sudoku Explainer
 * Copyright (C) 2006-2007 Nicolas Juillerat
 * Copyright (C) 2013-2020 Keith Corlett
 * Available under the terms of the Lesser General Public License (LGPL)
 */
package diuf.sudoku.solver.hinters.align2;

import diuf.sudoku.Settings;
import diuf.sudoku.Tech;
import static diuf.sudoku.TestHelp.*;
import static org.junit.Assert.*;
import diuf.sudoku.solver.AHint;
import org.junit.Test;


/**
 * Test AlignedOct.
 * @author Keith Corlett
 */
public final class Aligned8Exclusion_2HTest {

private static final boolean REPRODUCE = REPRODUCE_ALL_TESTS || false;

static {
	// the AlignedExclusion constructor reads this setting
	Settings.THE.set("isa8ehacked", true);
	Settings.THE.save();
}
private static final AlignedExclusion HINTER = new AlignedExclusion(Tech.AlignedOct);

/*
4#C:\Users\User\Documents\NetBeansProjects\DiufSudoku\top1465.d5.mt
..7..8.6.16..5..98...6.....7...6..35..4.37..239.2......5.4.3.2.47..2.3......16.4.
259,234,,13,49,,25,,134,,,23,37,,24,47,,,89,348,359,,479,12,12,57,34,,128,28,18,,49,49,,,56,18,,59,,,69,18,,,,56,,48,15,678,178,467,689,,1689,,78,,1678,,1679,,,16,89,,59,,58,16,28,238,2389,57,,,578,,79
   70	     25,169,133	31	 120	  1	Aligned Oct: G1, I1, I3, G4, G5, G6, G7, G9 (G7-7)
*/
private static final String PUZZLE
="..7..8.6.16..5..98...6.....7...6..35..4.37..239.2......5.4.3.2.47..2.3......16.4."+NL
+"259,234,,13,49,,25,,134,,,23,37,,24,47,,,89,348,359,,479,12,12,57,34,,128,28,18,,49,49,,,56,18,,59,,,69,18,,,,56,,48,15,678,178,467,689,,1689,,78,,1678,,1679,,,16,89,,59,,58,16,28,238,2389,57,,,578,,79";

/** Test AlignedOct. */
@Test
public void test() {
	final String testName = testName(this);
	HINTER.setIsEnabled(true);
	getHints(PUZZLE, HINTER, true);
	AHint hint = find(HINTS, "Aligned Oct: G1, I1, I3, G4, G5, G6, G7, G9");
if ( REPRODUCE ) {
//KEEP THESE: For use in creating test cases.
System.out.format("selectedCells : %s\n", diuf.sudoku.utils.Frmt.csv(hint.getAquaCells(0)));
System.out.format("oranges       : %s\n", hint.getOranges(0));
System.out.format("greens        : %s\n", hint.getGreens(0));
System.out.format("reds          : %s\n", hint.getReds(0));
System.out.format("redPots       : %s\n", hint.redPots);
System.out.format("clueHtml      : %s\n", hint.getClueHtml(true));
}
	checkAquaCells(hint, "G1, I1, I3, G4, G5, G6, G7, G9");
	assertEquals(newPots("G1-25, G4-49, G5-69, G6-678, G7-168, G9-578, I1-134, I3-34"), hint.getOranges(0));
	assertEquals(newPots("G2-47, G3-12"), hint.getGreens(0));
	assertEquals(newPots("G7-7"), hint.getReds(0));
	assertEquals(newPots("G7-7"), hint.redPots);
	checkClue(hint, "Look for an Aligned Oct at G1, I1, I3, G4, G5, G6, G7, G9");
	assertHintEqualsStoredHintsHtml(HINTER, hint, testName+".html", REPRODUCE);
}

}
