/*
 * Project: Sudoku Explainer
 * Copyright (C) 2006-2007 Nicolas Juillerat
 * Copyright (C) 2013-2020 Keith Corlett
 * Available under the terms of the Lesser General Public License (LGPL)
 */
package diuf.sudoku.solver.hinters.align2;

import diuf.sudoku.Settings;
import diuf.sudoku.Tech;
import static diuf.sudoku.TestHelp.*;
import static org.junit.Assert.*;
import diuf.sudoku.solver.AHint;
import org.junit.Test;


/**
 * Test the Aligned9Exclusion hinter.
 * @author Keith Corlett
 */
public final class Aligned9Exclusion_2HTest {

private static final boolean REPRODUCE = REPRODUCE_ALL_TESTS || false;

static {
	// the AlignedExclusion constructor reads this setting
	Settings.THE.set("isa9ehacked", true);
	Settings.THE.save();
}
private static final AlignedExclusion HINTER = new AlignedExclusion(Tech.AlignedNona);

private static final String PUZZLE
="..2...3...37..92.11......79283154.9..19...5.257.....13.2.....6.76.82.93...56.712." + NL
+"4689,459,,457,1478,156,,458,68,468,,,45,68,,,458,,,45,468,2345,368,28,468,,,,,,,,,67,,67,46,,,37,367,368,,48,,,,46,29,89,268,48,,,3489,,18,3459,1349,135,78,,457,,,14,,,15,,,45,3489,49,,,349,,,,48";

/*
63#C:\Users\User\Documents\NetBeansProjects\DiufSudoku\top1465.d5.mt
..2...3...37..92.11......79283154.9..19...5.257.....13.2.....6.76.82.93...56.712.
4689,459,,457,1478,156,,458,68,468,,,45,68,,,458,,,45,468,2345,368,28,468,,,,,,,,,67,,67,46,,,37,367,368,,48,,,,46,29,89,268,48,,,3489,,18,3459,1349,135,78,,457,,,14,,,15,,,45,3489,49,,,349,,,,48
   41	     32,107,236	38	 114	  1	Aligned Nona                  	Aligned Nona: D1, F1, D2, E2, D3, F3, E6, E7, E9 (F1-6)
*/

/** Test of Aligned9Exclusion.getHints method. */
@Test
public void test() {
	final String testName = testName(this);
	HINTER.setIsEnabled(true);
	getHints(PUZZLE, HINTER, true);
	AHint hint = find(HINTS, "Aligned Nona: D1, F1, D2, E2, D3, F3, E6, E7, E9");
////KEEP THESE: For use in creating test cases.
//System.out.format("selectedCells : %s\n", diuf.sudoku.utils.Frmt.csv(hint.getAquaCells(0)));
//System.out.format("oranges       : %s\n", hint.getOranges(0));
//System.out.format("greens        : %s\n", hint.getGreens(0));
//System.out.format("reds          : %s\n", hint.getReds(0));
//System.out.format("redPots       : %s\n", hint.redPots);
//System.out.format("clueHtml      : %s\n", hint.getClueHtml(true));
	checkAquaCells(hint, "D1, F1, D2, E2, D3, F3, E6, E7, E9");
	assertEquals(newPots("D1-457, D2-45, D3-2345, E2-68, E6-89, E7-1349, E9-349, F1-15, F3-28"), hint.getOranges(0));
	assertEquals(newPots("E1-1478, E3-368"), hint.getGreens(0));
	assertEquals(newPots("F1-6"), hint.getReds(0));
	assertEquals(newPots("F1-6"), hint.redPots);
	checkClue(hint, "Look for an Aligned Nona at D1, F1, D2, E2, D3, F3, E6, E7, E9");
	assertHintEqualsStoredHintsHtml(HINTER, hint, testName+".html", REPRODUCE);
}

}
