/*
 * Project: Sudoku Explainer
 * Copyright (C) 2006-2007 Nicolas Juillerat
 * Copyright (C) 2013-2020 Keith Corlett
 * Available under the terms of the Lesser General Public License (LGPL)
 */
package diuf.sudoku.solver.hinters.als;

import static diuf.sudoku.TestHelp.*;
import static org.junit.Assert.*;
import org.junit.Before;
import org.junit.Test;

import diuf.sudoku.Grid;
import diuf.sudoku.Idx;
import diuf.sudoku.Tech;
import diuf.sudoku.io.IO;
import diuf.sudoku.solver.AHint;
import diuf.sudoku.solver.accu.HintsAccumulator;
import diuf.sudoku.solver.accu.IAccumulator;
import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.LinkedList;


/**
 * AAlsHinterTest extends AAlsHinter to run tests on it's methods, including
 * it's otherwise-private getAlss and getRccs.
 *
 * @author Keith Corlett 2020-10-18
 */
public class AAlsHinterTest {

private static final String ALS_XZ_DOUBLE_LINKED
=".......3.6..5..18..83.1..2....9573...6.....92...........6...254.487...13..23....." + NL
+"24579,12579,14579,2468,24789,2489,45679,,5679,,279,479,,23479,2349,,,79,4579,,,46,,49,45679,,5679,1248,12,14,,,,,46,168,134578,,1457,148,348,1348,578,,,1345789,13579,14579,1248,2368,123468,578,47,1578,37,37,,18,89,189,,,,59,,,,26,256,69,,,159,159,,,46,456,6789,67,6789";
//C:/Users/User/Documents/SodukuPuzzles/HoDoKu/ALS-XZ_DoubleLinked_001.txt
//http://hodoku.sourceforge.net/en/tech_als.php
//ALS_XZ	ALS-XZ: B4, C4, B2, C2, I2 on 24 and - (A4-1, A5-1, A6-1, B1-2, B6-1, C1-4, C5-14, C6-14, E2-79, F2-9, I4-1)

/*
.......3.6..5..18..83.1..2....9573...6.....92...........6...254.487...13..23.....
24579,12579,14579,2468,24789,2489,45679,,5679,,279,479,,23479,2349,,,79,4579,,,46,,49,45679,,5679,1248,12,14,,,,,46,168,134578,,1457,148,348,1348,578,,,1345789,13579,14579,1248,2368,123468,578,47,1578,37,37,,18,89,189,,,,59,,,,26,256,69,,,159,159,,,46,456,6789,67,6789
*/

public AAlsHinterTest() {
}

@Before
public void setUp() {
	grid = new Grid(ALS_XZ_DOUBLE_LINKED);
}

///**
// * Test of getAlss method, of class AAlsHinter.
// */
//@Test
//public void testGetAlss() {
//	// nb: setUp() loads ALS_XZ_DOUBLE_LINKED into the static grid
//	final String testName = testName(this);
//	final String FILENAME = IO.HOME+"_alss.updated.txt";
//	AAlsHinterImpl h = new AAlsHinterImpl(Tech.ALS_XZ, true, true, true, false);
//	Als[] alss = h.getAlss(grid, grid.getIdxs());
//	if ( false ) {
//		// save the alss
//		try ( PrintWriter out = new PrintWriter(FILENAME) ) {
//			int i = 0;
//			for ( Als als : alss )
//				out.println(completeString(i++, als));
//			System.out.println("PRODUCED: "+FILENAME);
//		} catch (IOException ex) {
//			ex.printStackTrace(System.err);
//		}
//	} else {
//		// compare current alss with the saved file
//		try ( BufferedReader reader = new BufferedReader(new FileReader(FILENAME)) ) {
//			String line;
//			int i = 0;
//			while ( (line=reader.readLine()) != null ) {
//				assertEquals("mismatch als "+i+" "+line+"\t"+alss[i], line, completeString(i, alss[i]));
//				++i;
//			}
//			assertEquals("alss length saved "+i+" != actual "+alss.length, i, alss.length);
//		} catch (IOException ex) {
//			ex.printStackTrace(System.err);
//		}
//	}
//}
//
//	/**
//	 * Returns a "complete" ALS String, including the computeFields field.
//	 * We diff before and after files, to check that Idx ops same-as HdkSet.
//	 * <p>
//	 * NOTE: This is all done here outside of Als because it's only ever
//	 * used here, in a test-case; and so I can revert Als to original
//	 * without breaking this test-case.
//	 * <p>
//	 * NOTE: The code formatting is a bit weird. Live with it!
//	 *
//	 * @param i
//	 * @param als
//	 * @return
//	 */
//	String completeString(int i, Als als) {
//		StringBuilder sb = new StringBuilder(1024);
//		sb.append("ALS ").append(i)
//		  .append(" set=").append(als.set)
//		  .append(" maybes="); diuf.sudoku.Values.appendTo(sb, als.maybes)
////		  .append(" region=").append(als.region.id)
//		  .append(" vs="); appendIdxs(sb, als.vs)
//		  .append(" vBuds="); appendIdxs(sb, als.vBuds)
////		  .append(" vAll="); appendIdxs(sb, als.vAll)
////		  .append(" cells=").append(java.util.Arrays.toString(als.cells))
////		  .append(" buddies=").append(als.buddies)
//		  ;
//		return sb.toString();
//	}
//
//	/**
//	 * Append an Idx array to the given StringBuilder, only "used" (non-empty)
//	 * Idx's are appended.
//	 *
//	 * @param sb
//	 * @param idxs
//	 * @return
//	 */
//	private StringBuilder appendIdxs(StringBuilder sb, Idx[] idxs) {
//		boolean first = true;
//		Idx vs;
//		for ( int v=1; v<10; ++v ) {
//			vs = idxs[v];
//			if ( vs!=null && vs.any() ) {
//				if(first) first=false; else sb.append(' ');
//				sb.append(v).append('=');
//				vs.append(sb);
//			}
//		}
//		if ( first )
//			sb.append("NONE");
//		return sb;
//	}
//
//	// same but with HdkSet
//	private StringBuilder appendIdxs(StringBuilder sb, HdkSet[] idxs) {
//		boolean first = true;
//		HdkSet vs;
//		for ( int v=1; v<10; ++v ) {
//			vs = idxs[v];
//			if ( vs!=null && vs.any() ) {
//				if(first) first=false; else sb.append(' ');
//				sb.append(v).append('=');
//				vs.append(sb); // NOTE: This append method was added to HdkSet
//			}
//		}
//		if ( first )
//			sb.append("NONE");
//		return sb;
//	}


// ============================================================================

/**
 * Test of getRccs method, of class AAlsHinter.
 */
@Test
public void testGetRccs() {
	// nb: setUp() loads ALS_XZ_DOUBLE_LINKED into the static grid
	final String testName = testName(this);
	final String FILENAME = IO.HOME+"_rccs.updated.txt";
	// Tech, allowLockedSets, findRCCs, allowOverlaps, forwardOnly, useStartAndEnd
	AAlsHinterImpl h = new AAlsHinterImpl(Tech.ALS_XZ, true, true, true, true, false);
	Rcc[] rccs = h.getRccs(h.getAlss(grid, grid.getIdxs()));
	if ( true ) {
		// save the rccs
		try ( PrintWriter out = new PrintWriter(FILENAME) ) {
			for ( Rcc rcc : rccs )
				out.println(rcc);
		} catch (IOException ex) {
			ex.printStackTrace(System.err);
		}
	} else {
		// compare current alss with the saved file
		try ( BufferedReader reader = new BufferedReader(new FileReader(FILENAME)) ) {
			String line;
			int i = 0;
			while ( (line=reader.readLine()) != null ) {
				assertEquals("mismatch als "+i+" "+line+"\t"+rccs[i], line, rccs[i].toString());
				++i;
			}
			assertEquals("alss length saved "+i+" != actual "+rccs.length, i, rccs.length);
		} catch (IOException ex) {
			ex.printStackTrace(System.err);
		}
	}
}

/**
 * I'm a dummy implementation of AAlsHinter.
 * <p>
 * All I want to do is verify my ALSs and RCCs are same as HoDoKu's, and both
 * of them feed findHints, so findHints (normally the crux) doesn't matter.
 */
public class AAlsHinterImpl extends AAlsHinter {

	public AAlsHinterImpl(Tech tech, boolean allowLockedSets, boolean findRCCs
			, boolean allowOverlaps, boolean forwardOnly, boolean useStartAndEnd) {
		// Tech, allowLockedSets, findRCCs, allowOverlaps, forwardOnly, useStartAndEnd
		super(tech, allowLockedSets, findRCCs, allowOverlaps, forwardOnly, useStartAndEnd);
	}

	@Override
	public boolean findHints(Grid grid, Idx[] candidates, Rcc[] rccs
			, Als[] alss, IAccumulator accu) {
		return true;
	}
}

}
