/*
 * Project: Sudoku Explainer
 * Copyright (C) 2006-2007 Nicolas Juillerat
 * Copyright (C) 2013-2020 Keith Corlett
 * Available under the terms of the Lesser General Public License (LGPL)
 */
package diuf.sudoku.solver.hinters.als;

import static diuf.sudoku.TestHelp.*;
import static org.junit.Assert.*;
import org.junit.Test;
import diuf.sudoku.solver.AHint;
import diuf.sudoku.utils.MyLinkedHashSet;

/**
 * Test the HoDoKu ALS-XY-Chain hinter.
 * @author Keith Corlett 2020 Apr
 */
public final class AlsXyChainTest {

private static final boolean REPRODUCE = REPRODUCE_ALL_TESTS || false;

private static final AlsXyChain HINTER = new AlsXyChain();

private static final String ALS_XY_CHAIN
=".3.5.4.....4.3..2...58.6....2.149.566.925.1...51..8.9...3..58.........7.59..8...4" + NL
+"12789,,2678,,1279,,679,168,1789,1789,1678,,79,,17,5679,,15789,1279,17,,,279,,3479,34,1379,378,,78,,,,37,,,,47,,,,37,,348,378,347,,,367,67,,234,,23,127,1467,,4679,1679,,,16,129,1248,1468,268,3469,169,123,23569,,12359,,,267,367,,1237,236,136,";
//19#C:\\Users\\User\\Documents\\NetBeansProjects\\DiufSudoku\\top1465.d5.mt
//ALS-Chain: box 2, row 9, col H and col I (G2-7)

		
/** Test HoDoKu ALS-XY-Chain. */
@Test
public void test() {
	final String testName = testName(this);

	// nb: use the getHints which prepares the hinter.
//	getHints(ALS_XY_CHAIN, HINTER, true); // use when you're calling HintValidator.isValid (or not, sigh)
	getHints(ALS_XY_CHAIN, HINTER);

	// find my hint
	AHint hint = find(HINTS, "ALS-Chain: box 2, row 9, col H, col I");
////KEEP THESE: For use in creating test cases.
//System.out.format("aquaCells  : %s\n", diuf.sudoku.utils.Frmt.csv(hint.getAquaCells(0)));
//System.out.format("oranges    : %s\n", hint.getOranges(0));
//System.out.format("greens     : %s\n", hint.getGreens(0));
//System.out.format("reds       : %s\n", hint.getReds(0));
//System.out.format("redPots    : %s\n", hint.redPots);
//System.out.format("clueHtml   : %s\n", hint.getClueHtml(true));
//System.out.format("bases      : %s\n", diuf.sudoku.utils.Frmt.csv(hint.getBases()));
//System.out.format("covers     : %s\n", diuf.sudoku.utils.Frmt.csv(hint.getCovers()));
	checkClue(hint, "Look for a ALS-Chain in box 2, row 9, col H, col I");
	assertEquals(grid.getCSVSet("D2, F2, C9, D9, F9, G9, H1, H7, H9, I1, I2, I3, I6, I7, I8"), hint.getAquaCells(0));
	assertEquals(newPots("C9-267, D2-9, D9-367, F2-1, F9-1237, G9-236, H1-168, H7-16, H9-136, I1-189, I2-1589, I3-139, I6-23, I7-129, I8-12359"), hint.getOranges(0));
	assertEquals(null, hint.getGreens(0));
	assertEquals(newPots("G2-7"), hint.getReds(0));
	assertEquals(newPots("G2-7"), hint.redPots);
	assertArrayEquals(regions("box 2, col H"), hint.getBasesArray());
	assertArrayEquals(regions("row 9, col I"), hint.getCoversArray());
	assertHintEqualsStoredHintsHtml(this, hint, testName+".html", REPRODUCE);
}

}