/*
 * Project: Sudoku Explainer
 * Copyright (C) 2006-2007 Nicolas Juillerat
 * Copyright (C) 2013-2020 Keith Corlett
 * Available under the terms of the Lesser General Public License (LGPL)
 */
package diuf.sudoku.solver.hinters.als;

import static diuf.sudoku.TestHelp.*;
import static org.junit.Assert.*;
import org.junit.Test;
import diuf.sudoku.solver.AHint;
import diuf.sudoku.utils.MyLinkedHashSet;

/**
 * Test the HoDoKu ALS-Wing hinter.
 * @author Keith Corlett 2020 Apr
 */
public final class AlsXyWingTest {

private static final boolean REPRODUCE = REPRODUCE_ALL_TESTS || false;

private static final AlsXyWing HINTER = new AlsXyWing();

private static final String ALS_WING
="4.6....7......146....684...23.....4....34.......8.2953.9.......3.2.5761.5...2...4" + NL
+",125,,25,39,359,12358,,12589,89,258,3589,257,379,,,,259,179,1257,359,,,,1235,239,1259,,,589,57,1679,569,178,,1678,16789,1567,589,,,569,1278,28,12678,167,1467,147,,167,,,,,678,,14,14,36,368,23578,238,2578,,48,,49,,,,,89,,1678,17,19,,368,378,389,";
//C:/Users/User/Documents/SodukuPuzzles/HoDoKu/ALS-Wing_001.txt
//ALS_Wing	ALS-Wing: box 9, col H and row 9 (G7-3, H3-2)

/** Test ALS-Wing. */
@Test
public void test() {
	final String testName = testName(this);

	// nb: use the getHints which prepares the hinter.
//	getHints(ALS_WING, HINTER, true); // use when you're calling HintValidator.isValid (or not, sigh)
	getHints(ALS_WING, HINTER);

	// find my hint
	// nb: There's 2 hints matching hint.toString, and I need the one with the
	//     " (G7-3, H3-2)", so I changed find to call toFullString!
	AHint hint = find(HINTS, "ALS-Wing: box 9, col H, row 9 (G7-3, H3-2)");
////KEEP THESE: For use in creating test cases.
//System.out.format("aquaCells  : %s\n", diuf.sudoku.utils.Frmt.csv(hint.getAquaCells(0)));
//System.out.format("clueHtml   : %s\n", hint.getClueHtml(true));
//System.out.format("oranges    : %s\n", hint.getOranges(0));
//System.out.format("greens     : %s\n", hint.getGreens(0));
//System.out.format("reds       : %s\n", hint.getReds(0));
//System.out.format("redPots    : %s\n", hint.redPots);
//System.out.format("bases      : %s\n", diuf.sudoku.utils.Frmt.csv(hint.getBases()));
//System.out.format("covers     : %s\n", diuf.sudoku.utils.Frmt.csv(hint.getCovers()));
	checkClue(hint, "Look for a ALS-Wing in box 9, col H, row 9");
	assertEquals(grid.getCSVSet("H7, I8, G9, H9, H5, C9, D9"), hint.getAquaCells(0));
	assertEquals(newPots("C9-17, D9-19, G9-78, H5-8, H7-8, H9-89, I8-89"), hint.getOranges(0));
	assertEquals(null, hint.getGreens(0));
	assertEquals(newPots("G7-3, H3-2"), hint.getReds(0));
	assertEquals(newPots("G7-3, H3-2"), hint.redPots);
	assertArrayEquals(regions("box 9, row 9"), hint.getBasesArray());
	assertArrayEquals(regions("col H"), hint.getCoversArray());
	assertHintEqualsStoredHintsHtml(this, hint, testName+".html", REPRODUCE);
}

}