/*
 * Project: Sudoku Explainer
 * Copyright (C) 2006-2007 Nicolas Juillerat
 * Copyright (C) 2013-2020 Keith Corlett
 * Available under the terms of the Lesser General Public License (LGPL)
 */
package diuf.sudoku.solver.hinters.als;

import static diuf.sudoku.TestHelp.*;
import static org.junit.Assert.*;
import org.junit.Test;
import diuf.sudoku.solver.AHint;
import diuf.sudoku.utils.MyLinkedHashSet;

/**
 * Test the WWing hinter.
 * @author Keith Corlett 2020 Apr
 */
public final class AlsXzTest {

private static final boolean REPRODUCE = REPRODUCE_ALL_TESTS || false;

private static final AlsXz HINTER = new AlsXz();


private static final String ALS_XZ_SINGLE_LINKED
="7.....4...2..7..83..3..8..9...5.63...6..2..9...1..7..6...3.29...3..4..6...9..1.35" + NL
+",189,568,1269,1369,359,,125,12,19,,56,149,,45,156,,,146,145,,126,156,,1267,127,,289,789,247,,189,,,1247,12478,35,,457,18,,34,178,,1478,23,89,,489,39,,258,245,,168,1578,678,,568,,,147,1478,1258,,257,789,,59,1278,,1278,2468,478,,678,68,,278,,";
//1#C:/Users/User/Documents/NetBeansProjects/DiufSudoku/top1465.d5.mt	7.....4...2..7..8...3..8..9...5..3...6..2..9...1..7..6...3..9...3..4..6...9..1..5
//37   	     15,169,400	28	 159	  1	ALS-XZ                        	ALS-XZ: A4, B6, A6, E6 on 2 and 8 (B4-8)

/** Test Single-linked ALS-XZ. */
@Test
public void test() {
	final String testName = testName(this);

	// nb: use the getHints which prepares the hinter.
//	getHints(ALS_XZ_SINGLE_LINKED, HINTER, true); // for when invalid is called
	getHints(ALS_XZ_SINGLE_LINKED, HINTER);

	// find my hint
	AHint hint = find(HINTS, "ALS-XZ: in box 5 and row 6 on 4 and 9");
////KEEP THESE: For use in creating test cases.
//System.out.format("clueHtml   : %s\n", hint.getClueHtml(true));
//System.out.format("aquaCells  : %s\n", diuf.sudoku.utils.Frmt.csv(hint.getAquaCells(0)));
//System.out.format("oranges    : %s\n", hint.getOranges(0));
//System.out.format("greens     : %s\n", hint.getGreens(0));
//System.out.format("reds       : %s\n", hint.getReds(0));
//System.out.format("redPots    : %s\n", hint.redPots);
//System.out.format("bases      : %s\n", diuf.sudoku.utils.Frmt.csv(hint.getBases()));
//System.out.format("covers     : %s\n", diuf.sudoku.utils.Frmt.csv(hint.getCovers()));
	checkClue(hint, "Look for a ALS-XZ in box 5 and row 6 on 4 and 9");
	assertEquals(grid.getCSVSet("F5, E6, B6, D6"), hint.getAquaCells(0));
	assertEquals(newPots("B6-89, D6-489, E6-39, F5-34"), hint.getOranges(0));
	assertEquals(null, hint.getGreens(0));
	assertEquals(newPots("G6-8"), hint.getReds(0));
	assertEquals(newPots("G6-8"), hint.redPots);
	assertArrayEquals(regions("box 5"), hint.getBasesArray());
	assertArrayEquals(regions("row 6"), hint.getCoversArray());
	assertHintEqualsStoredHintsHtml(this, hint, testName+".html", REPRODUCE);
}



private static final String ALS_XZ_DOUBLE_LINKED
=".......3.6..5..18..83.1..2....9573...6.....92...........6...254.487...13..23....." + NL
+"24579,12579,14579,2468,24789,2489,45679,,5679,,279,479,,23479,2349,,,79,4579,,,46,,49,45679,,5679,1248,12,14,,,,,46,168,134578,,1457,148,348,1348,578,,,1345789,13579,14579,1248,2368,123468,578,47,1578,37,37,,18,89,189,,,,59,,,,26,256,69,,,159,159,,,46,456,6789,67,6789";
//C:/Users/User/Documents/SodukuPuzzles/HoDoKu/ALS-XZ_DoubleLinked_001.txt
//http://hodoku.sourceforge.net/en/tech_als.php
//ALS_XZ	ALS-XZ: B4, C4, B2, C2, I2 on 24 and - (A4-1, A5-1, A6-1, B1-2, B6-1, C1-4, C5-14, C6-14, E2-79, F2-9, I4-1)

/*
.......3.6..5..18..83.1..2....9573...6.....92...........6...254.487...13..23.....
24579,12579,14579,2468,24789,2489,45679,,5679,,279,479,,23479,2349,,,79,4579,,,46,,49,45679,,5679,1248,12,14,,,,,46,168,134578,,1457,148,348,1348,578,,,1345789,13579,14579,1248,2368,123468,578,47,1578,37,37,,18,89,189,,,,59,,,,26,256,69,,,159,159,,,46,456,6789,67,6789
*/

/** Test Double-linked ALS-XZ (with 2 X's (RCs) and no Z value) */
@Test
public void test2() {
	final String testName = testName(this);

	// nb: use the getHints which prepares the hinter.
//	getHints(ALS_XZ_DOUBLE_LINKED, HINTER, true); // for when invalid is called
	getHints(ALS_XZ_DOUBLE_LINKED, HINTER);

	// find my hint
	AHint hint = find(HINTS, "ALS-XZ: in box 4 and row 2 on 2 and 4");
////KEEP THESE: For use in creating test cases.
//System.out.format("aquaCells  : %s\n", diuf.sudoku.utils.Frmt.csv(hint.getAquaCells(0)));
//System.out.format("oranges    : %s\n", hint.getOranges(0));
//System.out.format("greens     : %s\n", hint.getGreens(0));
//System.out.format("reds       : %s\n", hint.getReds(0));
//System.out.format("redPots    : %s\n", hint.redPots);
//System.out.format("clueHtml   : %s\n", hint.getClueHtml(true));
//System.out.format("bases      : %s\n", diuf.sudoku.utils.Frmt.csv(hint.getBases()));
//System.out.format("covers     : %s\n", diuf.sudoku.utils.Frmt.csv(hint.getCovers()));
	checkClue(hint, "Look for a ALS-XZ in box 4 and row 2 on 2 and 4");
	assertEquals(grid.getCSVSet("B4, C4, B2, C2, I2"), hint.getAquaCells(0));
	assertEquals(newPots("B2-279, B4-12, C2-479, C4-14, I2-79"), hint.getOranges(0));
	assertEquals(null, hint.getGreens(0));
	assertEquals(newPots("A4-1, A5-1, A6-1, B1-2, B6-1, C1-4, C5-14, C6-14, E2-79, F2-9, I4-1"), hint.getReds(0));
	assertEquals(newPots("A4-1, A5-1, A6-1, B1-2, B6-1, C1-4, C5-14, C6-14, E2-79, F2-9, I4-1"), hint.redPots);
	assertArrayEquals(regions("box 4"), hint.getBasesArray());
	assertArrayEquals(regions("row 2"), hint.getCoversArray());
	assertHintEqualsStoredHintsHtml(this, hint, testName+".html", REPRODUCE);
}

}
