/*
 * Project: Sudoku Explainer
 * Copyright (C) 2006-2007 Nicolas Juillerat
 * Copyright (C) 2013-2020 Keith Corlett
 * Available under the terms of the Lesser General Public License (LGPL)
 */
package diuf.sudoku.solver.hinters.bug;

import static org.junit.Assert.*;
import static diuf.sudoku.TestHelp.*;
import diuf.sudoku.Grid.Cell;
import diuf.sudoku.Pots;
import diuf.sudoku.Values;
import diuf.sudoku.solver.AHint;
import diuf.sudoku.solver.hinters.AHinter;
import org.junit.Test;


/**
 * Test the BivalueUniversalGrave hinter.
 * @author Keith Corlett 2019 Apr
 */
public final class BivalueUniversalGraveTest {
	
private static final boolean REPRODUCE = REPRODUCE_ALL_TESTS || false;

private static final AHinter HINTER = new BivalueUniversalGrave();

private static final String BUG1
="14358.9.28..49..3.976321.8..146..39863..19..7..9.436..3.1.64.5946.9....35971382.." + NL
+",,,,,67,,67,,,25,25,,,67,17,,16,,,,,,,45,,45,27,,,,57,25,,,,,,58,28,,,45,24,,27,58,,278,,,,12,15,,28,,27,,,78,,,,,28,,57,25,18,17,,,,,,,,,46,46";
//82#C:\Users\User\Documents\SodukuPuzzles\MagicTour\top1465.d3.mt
// BUG type 1: D6 (D6-78)"

/** Test Bivalue Universal Grave type 1. */
@Test
public void testBUGtype1() {
	final String testName = testName(this);
	getHints(BUG1, HINTER);
	AHint hint = find(HINTS, "BUG type 1: D6");
	String id = "D6";
	Cell d6 = grid.get(id);
	checkAquaCells(hint, id);
	assertEquals(new Pots(d6, 2), hint.getGreens(0));
	assertEquals(new Pots(d6, new Values(7,8)), hint.getReds(0));
	assertEquals(new Pots(d6, new Values(7,8)), hint.redPots);
	assertEquals("Look for a BUG type 1", hint.getClueHtml(true));
	assertHintEqualsStoredHintsHtml(this, hint, testName+".html", REPRODUCE);
}


private static final String BUG2
="237.948.6695..7.34..46......5.41.3...43..2..5.6.3584..4792..5..5.6....4.32..4.69." + NL
+",,,15,,,,15,,,,,18,28,,12,,,18,18,,,23,35,79,25,79,79,,28,,,69,,67,28,18,,,79,67,,19,68,,79,,12,,,,,27,19,,,,,68,136,,18,13,,18,,89,37,19,27,,23,,,18,57,,15,,,178";
//111#C:\Users\User\Documents\NetBeansProjects\DiufSudoku\top1465.d3.mt
// BUG type 2: F7, I9 on 1 (F9-1, H7-1, I7-1)

/** Test Bivalue Universal Grave type 2. */
@Test
public void testBUGtype2() {
	final String testName = testName(this);
	getHints(BUG2, HINTER);
	AHint hint = find(HINTS, "BUG type 2: F7, I9 on 1");
	checkAquaCells(hint, "F7, I9");
	assertEquals(newPots(1, "F7","I9"), hint.getGreens(0));
	assertEquals(newPots(1, "H7","I7","F9"), hint.getReds(0));
	assertEquals(newPots(1, "H7","I7","F9"), hint.redPots);
	assertEquals("Look for a BUG type 2", hint.getClueHtml(true));
	assertHintEqualsStoredHintsHtml(this, hint, testName+".html", REPRODUCE);
}


private static final String BUG4
="6791..28.4532..1972..7..64.7.583..6.89.5..73.3624.7.5.52.67.3199.632157.1379.5.26" + NL
+",,,,45,34,,,35,,,,,68,68,,,,,18,18,,59,39,,,35,,14,,,,29,49,,124,,,14,,16,26,,,124,,,,,19,,89,,18,,,48,,,48,,,,,48,,,,,,,48,,,,,48,,48,,";
//C:\Users\User\Documents\SodukuPuzzles\MagicTour\top1465.d3.0182.BUG4.mt.txt
//BUG type 4: I4, I5 on 2

/** Test Bivalue Universal Grave type 4. */
@Test
public void testBUGtype4() {
	final String testName = testName(this);
	getHints(BUG4, HINTER);
	AHint hint = find(HINTS, "BUG type 4: I4, I5 on 2");
////KEEP THESE: For use in creating test cases.
//System.out.format("aquaCells  : %s\n", diuf.sudoku.utils.Frmt.csv(hint.getAquaCells(0)));
//System.out.format("oranges    : %s\n", hint.getOranges(0));
//System.out.format("greens     : %s\n", hint.getGreens(0));
//System.out.format("reds       : %s\n", hint.getReds(0));
//System.out.format("redPots    : %s\n", hint.redPots);
//System.out.format("bases      : %s\n", diuf.sudoku.utils.Frmt.csv(hint.getBases()));
//System.out.format("covers     : %s\n", diuf.sudoku.utils.Frmt.csv(hint.getCovers()));
	checkAquaCells(hint, "I4, I5");
	assertEquals(newPots("I4-2 I5-2"), hint.getOranges(0));
	assertEquals(null, hint.getGreens(0));
	assertEquals(newPots("I4-1 I5-4"), hint.getReds(0));
	assertEquals(newPots("I4-1 I5-4"), hint.redPots);
	assertEquals("Look for a BUG type 4", hint.getClueHtml(true));
	assertEquals(grid.boxs[5], hint.getBases().get(0));
	assertHintEqualsStoredHintsHtml(this, hint, testName+".html", REPRODUCE);
}

}