/*
 * Project: Sudoku Explainer
 * Copyright (C) 2006-2007 Nicolas Juillerat
 * Copyright (C) 2013-2020 Keith Corlett
 * Available under the terms of the Lesser General Public License (LGPL)
 */
package diuf.sudoku.solver.hinters.chain;

import diuf.sudoku.Pots;
import static diuf.sudoku.TestHelp.*;
import static org.junit.Assert.*;
import diuf.sudoku.solver.AHint;
import org.junit.Test;


/**
 * Test the Chainer hinter: BidirectionalCycleHint's
 * @author Keith Corlett 2018 Apr
 */
public final class BidirectionalCycleTest {
	
private static final boolean REPRODUCE = REPRODUCE_ALL_TESTS || false;

// finds Unary Forcing Chains and Bidirectional Cycles
private static final boolean isAggregate = false;
private static final UnaryChainer UNARY_CHAINER = new UnaryChainer(isAggregate);

//private static final String XY_CYCLE
//="7.....4...2..7..8...3..8..9...5.63...6..2..9...1..7..6...3.29...3..4..6...9..1.35" + NL
//+",89,68,26,13,59,,25,13,19,,56,149,,34,56,,13,14,145,,26,15,,267,27,,89,789,247,,189,,,14,2478,358,,457,18,,34,1578,,478,2359,589,,48,39,,258,245,,1568,1578,78,,568,,,147,478,158,,2578,789,,59,178,,278,2468,48,,78,68,,278,,";
////1#C:\Users\User\Documents\NetBeansProjects\DiufSudoku\top1465.d3.mt
//// Bidirectional XY Cycle: F2, E1, E6, A6, C4, C5, F5 (I5-4)
///*
//7.....4...2..7..8...3..8..9...5.63...6..2..9...1..7..6...3.29...3..4..6...9..1.35
//,89,68,26,13,59,,25,13,19,,56,149,,34,56,,13,14,145,,26,15,,267,27,,89,789,247,,189,,,14,2478,358,,457,18,,34,1578,,478,2359,589,,48,39,,258,245,,1568,1578,78,,568,,,147,478,158,,2578,789,,59,178,,278,2468,48,,78,68,,278,,
//*/
//
///** Test Bidirectional XY Cycle. */
//@Test
//public void testXYCycle() {
//	final String testName = testName(this);
//	getHints(XY_CYCLE, HINTER);
//	AHint hint = find(HINTS, "Bidirectional XY Cycle: F2, E1, E6, A6, C4, C5, F5");
//	checkSelectedCells(hint, "F2","E1","E6","A6","C4","C5","F5");
//	assertEquals(newPots("A6-3 C4-2 C5-4 E1-3 F2-4"), hint.getGreens(0));
//	assertEquals(newPots("A6-2 C4-4 E6-3 F2-3 F5-4 I5-4"), hint.getReds(0));
//	assertEquals(new Pots(grid.get("I5"), 4), hint.redPots);
//	checkClue(hint, "Look for a Bidirectional XY Cycle at <b>F2</b>");
//	assertHintEqualsStoredHintsHtml(HINTER, hint, testName+".html", REPRODUCE);
//}
private static final String XY_CYCLE
//ORIG
="7.....4...2..7..8...3..8..9...5.63...6..2..9...1..7..6...3.29...3..4..6...9..1.35" + NL
+",89,68,26,13,59,,25,13,19,,56,149,,34,56,,13,14,145,,26,15,,267,27,,89,789,247,,189,,,14,2478,358,,457,18,,34,1578,,478,2359,589,,48,39,,258,245,,1568,1578,78,,568,,,147,478,158,,2578,789,,59,178,,278,2468,48,,78,68,,278,,";
/*
7.....4...2..7..8...3..8..9...5.63...6..2..9...1..7..6...3.29...3..4..6...9..1.35
,89,68,26,13,59,,25,13,19,,56,149,,34,56,,13,14,145,,26,15,,267,27,,89,789,247,,189,,,14,2478,358,,457,18,,34,1578,,478,2359,589,,48,39,,258,245,,1568,1578,78,,568,,,147,478,158,,2578,789,,59,178,,278,2468,48,,78,68,,278,,
*/
// 193125288 Bidirectional XY Cycle: A6, A5, F5, C5, C4
/** Test Bidirectional XY Cycle. */
@Test
public void testXYCycle() {
	final String testName = testName(this);
	UNARY_CHAINER.cleanUp(); // clear the hints cache!
	getHints(XY_CYCLE, UNARY_CHAINER);
	AHint hint = find(HINTS, "Bidirectional XY Cycle: F2, E1, E6, A6, C4, C5, F5");
////KEEP THESE: For use in creating test cases.
//System.out.format("aquaCells : %s\n", diuf.sudoku.utils.Frmt.csv(hint.getAquaCells(0)));
//System.out.format("oranges   : %s\n", hint.getOranges(0));
//System.out.format("greens    : %s\n", hint.getGreens(0));
//System.out.format("reds      : %s\n", hint.getReds(0));
//System.out.format("redPots   : %s\n", hint.redPots);
//System.out.format("clueHtml  : %s\n", hint.getClueHtml(true));
//System.out.format("regions   : %s\n", diuf.sudoku.utils.Frmt.csv((Object[])hint.getRegions()));
	checkAquaCells(hint, "F2, E1, E6, A6, C4, C5, F5");
	assertEquals(Pots.EMPTY, hint.getOranges(0));
	assertEquals(newPots("A6-3, C4-2, C5-4, E1-3, F2-4"), hint.getGreens(0));
	assertEquals(newPots("A6-2, C4-4, E6-3, F2-3, F5-4, I5-4"), hint.getReds(0));
	assertEquals(newPots("I5-4"), hint.redPots);
	checkClue(hint, "Look for a Bidirectional XY Cycle at <b>F2</b>");
	assertHintEqualsStoredHintsHtml(UNARY_CHAINER, hint, testName+".html", REPRODUCE||false);
}


private static final String X_CYCLE
=".4635....8..47.....37.2.45.37..8.6.4..8.34.9.4.......8..4.67...7.35...4..8..437.2" + NL
+"129,,,,,19,1289,78,179,,159,125,,,169,19,1236,1369,19,,,1689,,1689,,,169,,,1259,129,,259,,12,,56,1256,,67,,,125,,157,,1569,125,12679,19,12569,135,37,,129,129,,28,,,13589,138,1359,,1269,,,19,28,89,,169,56,,159,19,,,,16,";
//26#C:\Users\User\Documents\NetBeansProjects\DiufSudoku\top1465.d3.mt
// Bidirectional X Cycle: C9, D9, E8, E6, B6, C4 (D6-9, F6-9)
/*
.4635....8..47.....37.2.45.37..8.6.4..8.34.9.4.......8..4.67...7.35...4..8..437.2
129,,,,,19,1289,78,179,,159,125,,,169,19,1236,1369,19,,,1689,,1689,,,169,,,1259,129,,259,,12,,56,1256,,67,,,125,,157,,1569,125,12679,19,12569,135,37,,129,129,,28,,,13589,138,1359,,1269,,,19,28,89,,169,56,,159,19,,,,16,
*/

/** Test Bidirectional X Cycle. */
@Test
public void testXCycle() {
	final String testName = testName(this);
	UNARY_CHAINER.cleanUp(); // clear the hints cache!
	getHints(X_CYCLE, UNARY_CHAINER);
	AHint hint = find(HINTS, "Bidirectional X Cycle: C9, D9, E8, E6, B6, C4");
////KEEP THESE: For use in creating test cases.
//System.out.format("selectedCells : %s\n", diuf.sudoku.utils.Frmt.csv(hint.getAquaCells(0)));
//System.out.format("oranges       : %s\n", hint.getOranges(0));
//System.out.format("greens        : %s\n", hint.getGreens(0));
//System.out.format("reds          : %s\n", hint.getReds(0));
//System.out.format("redPots       : %s\n", hint.redPots);
//System.out.format("clueHtml      : %s\n", hint.getClueHtml(true));
//System.out.format("regions       : %s\n", diuf.sudoku.utils.Frmt.csv((Object[])hint.getRegions()));
	checkAquaCells(hint, "C9, D9, E8, E6, B6, C4");
	assertEquals(Pots.EMPTY, hint.getOranges(0));
	assertEquals(newPots("C4-9, D9-9, E6-9"), hint.getGreens(0));
	assertEquals(newPots("B6-9, C9-9, D6-9, E8-9, F6-9"), hint.getReds(0));
	assertEquals(newPots("D6-9, F6-9"), hint.redPots);
	checkClue(hint, "Look for a Bidirectional X Cycle on <b>9</b>");
	assertHintEqualsStoredHintsHtml(UNARY_CHAINER, hint, testName+".html", REPRODUCE||false);
}


private static final String Y_CYCLE
="7.....4...2..7..8.4.3..8..9.7.5.631..6.12.897..1..7..6...3.29...3..4.16..49..1.35" + NL
+",89,68,26,13,59,,25,13,19,,56,49,,34,56,,13,,15,,26,15,,267,27,,89,,24,,89,,,,24,35,,45,,,34,,,,23,89,,48,39,,25,245,,16,15,78,,56,,,47,48,58,,2578,79,,59,,,28,26,,,78,68,,27,,";
//1#C:\Users\User\Documents\NetBeansProjects\DiufSudoku\top1465.d3.mt
// Bidirectional Y Cycle: A5, A8, A4, A2, B3, C2, G2, G6, A6
/*
7.....4...2..7..8.4.3..8..9.7.5.631..6.12.897..1..7..6...3.29...3..4.16..49..1.35
,89,68,26,13,59,,25,13,19,,56,49,,34,56,,13,,15,,26,15,,267,27,,89,,24,,89,,,,24,35,,45,,,34,,,,23,89,,48,39,,25,245,,16,15,78,,56,,,47,48,58,,2578,79,,59,,,28,26,,,78,68,,27,,
*/

/** Test Bidirectional Y Cycle. */
@Test
public void testYCycle() {
	final String testName = testName(this);
	UNARY_CHAINER.cleanUp(); // clear the hints cache!
	getHints(Y_CYCLE, UNARY_CHAINER);
	AHint hint = find(HINTS, "Bidirectional Y Cycle: C4, A6, A5, A8, A4, A2, B3, C2, G2, G6, I4");
////KEEP THESE: For use in creating test cases.
//System.out.format("selectedCells : %s\n", diuf.sudoku.utils.Frmt.csv(hint.getAquaCells(0)));
//System.out.format("oranges       : %s\n", hint.getOranges(0));
//System.out.format("greens        : %s\n", hint.getGreens(0));
//System.out.format("reds          : %s\n", hint.getReds(0));
//System.out.format("redPots       : %s\n", hint.redPots);
//System.out.format("clueHtml      : %s\n", hint.getClueHtml(true));
//System.out.format("regions       : %s\n", diuf.sudoku.utils.Frmt.csv((Object[])hint.getRegions()));
	checkAquaCells(hint, "C4, A6, A5, A8, A4, A2, B3, C2, G2, G6, I4");
	assertEquals(Pots.EMPTY, hint.getOranges(0));
	assertEquals(newPots("A2-1, A4-9, A5-5, A6-3, A8-8, B3-5, C2-6, C4-2, G2-5, G6-2, I4-4"), hint.getGreens(0));
	assertEquals(newPots("A2-9, A4-8, A5-3, A6-2, A8-5, B3-1, B7-5, C2-5, C4-4, C5-5, G2-6, G6-5, H6-2, I4-2"), hint.getReds(0));
	assertEquals(newPots("B7-5, C5-5, H6-2"), hint.redPots);
	checkClue(hint, "Look for a Bidirectional Y Cycle at <b>I4</b>");
	assertHintEqualsStoredHintsHtml(this, hint, testName+".html", REPRODUCE||false);
}

}