/*
 * Project: Sudoku Explainer
 * Copyright (C) 2006-2007 Nicolas Juillerat
 * Copyright (C) 2013-2020 Keith Corlett
 * Available under the terms of the Lesser General Public License (LGPL)
 */
package diuf.sudoku.solver.hinters.chain;

import diuf.sudoku.Tech;
import org.junit.Test;
import static diuf.sudoku.TestHelp.*;
import diuf.sudoku.solver.hinters.AHinter;


/**
 * Test the Chainer hinter: RegionReductionHint's
 * @author Keith Corlett 2018 Apr
 */
public final class RegionReductionTest {

private static final boolean REPRODUCE = REPRODUCE_ALL_TESTS || false;

private static final boolean isAggregate = false;


private static final String STATIC
="7.....4...2..7..8...3..8..9...5.63...6..2..9...1..7..6...3..9...3..4..6...9..1.35" + NL
+",89,68,26,13,359,,25,123,1569,,56,149,,349,156,,13,146,145,,26,15,,267,127,,289,789,247,,189,,,147,12478,358,,457,18,,34,1578,,478,2359,589,,48,39,,258,245,,1568,1578,5678,,568,25,,1247,2478,1258,,2578,789,,259,1278,,1278,2468,478,,78,68,,278,,";
// Multiple XY Region Chain: 2 in box 6 so I1-2

/** Test Static Region Reduction. */
@Test
public void testStatic() {
	final String testName = testName(this);
	//							 M,D,N,L,A
	AHinter hinter = new MultipleChainer(Tech.MultipleChain, isAggregate);
	assertHintEqualsStoredHintsHtml(this, STATIC, hinter, testName+".html"
			, "Region Reduction Chain: 5 in col F so B9-7", REPRODUCE);
}


private static final String DYNAMIC
="7.....4...2..7..8...3..8..9...5..3...6..2..9...1..7..6...3..9...3..4..6...9..1.35" + NL
+",1589,568,1269,13569,23569,,125,123,14569,,456,149,,3459,156,,13,1456,145,,1246,156,,12567,1257,,2489,4789,2478,,1689,46,,1247,12478,358,,457,18,,34,1578,,1478,234589,4589,,489,389,,258,245,,12568,1578,25678,,568,256,,1247,12478,1258,,2578,2789,,259,1278,,1278,2468,478,,2678,68,,278,,";
// Dynamic Chain	Dynamic Multiple XY Region Chain: 9 in row 6 so E6-8 (E6-8)

/** Test Dynamic Region Reduction. */
@Test
public void testDynamic() {
	final String testName = testName(this);
	//							 M,D,N,L,A
	AHinter hinter = new MultipleChainer(Tech.DynamicChain, isAggregate);
	assertHintEqualsStoredHintsHtml(this, DYNAMIC, hinter, testName+".html"
			, "Region Reduction Chain: 9 in row 6 so E6-8", REPRODUCE);
}

}