/*
 * Project: Sudoku Explainer
 * Copyright (C) 2006-2007 Nicolas Juillerat
 * Copyright (C) 2013-2020 Keith Corlett
 * Available under the terms of the Lesser General Public License (LGPL)
 */
package diuf.sudoku.solver.hinters.chain;

import static org.junit.Assert.*;
import static diuf.sudoku.TestHelp.*;
import org.junit.Test;
import diuf.sudoku.solver.AHint;
import diuf.sudoku.solver.hinters.AHinter;


/**
 * Test the Chainer hinter: UnaryChainHint's
 * @author Keith Corlett 2018 Apr
 */
public final class UnaryChainTest {

private static final boolean REPRODUCE = REPRODUCE_ALL_TESTS || false;

private static final boolean isAggregate = false;
private static final AHinter HINTER = new UnaryChainer(isAggregate);

private static final String UNARY_XY
="7.....4...2..7..8...3..8..9...5..3...6..2..9...1..7..6...3..9...3..4..6...9..1.35" + NL
+",189,68,26,13,3569,,25,123,1569,,456,149,,3459,156,,13,1456,145,,246,15,,267,127,,289,789,247,,1689,46,,147,12478,358,,457,18,,34,1578,,478,234589,4589,,48,39,,258,245,,1568,1578,5678,,568,25,,1247,12478,1258,,2578,789,,259,1278,,1278,2468,478,,678,68,,278,,";

/** Test UnaryChainHintXY. */
@Test
public void testUnaryChainHintXY() {
	final String testName = testName(this);
	getHints(UNARY_XY, HINTER);
	AHint hint = find(HINTS, "Unary XY Chain: A6-8");
////KEEP THESE: For use in creating test cases.
//System.out.format("aquaCells  : %s\n", diuf.sudoku.utils.Frmt.csv(hint.getAquaCells(0)));
//System.out.format("oranges    : %s\n", hint.getOranges(0));
//System.out.format("greens     : %s\n", hint.getGreens(0));
//System.out.format("reds       : %s\n", hint.getReds(0));
//System.out.format("redPots    : %s\n", hint.redPots);
//System.out.format("clueHtml   : %s\n", hint.getClueHtml(true));
//System.out.format("bases      : %s\n", diuf.sudoku.utils.Frmt.csv(hint.getBases()));
//System.out.format("covers     : %s\n", diuf.sudoku.utils.Frmt.csv(hint.getCovers()));
	checkAquaCells(hint, "A6");
	assertEquals(newPots("A6-8"), hint.getOranges(0));
	assertEquals(newPots("A5-3, D6-8, F5-4"), hint.getGreens(0));
	assertEquals(newPots("A6-3, D6-4, F5-3"), hint.getReds(0));
	assertEquals(newPots("A6-8"), hint.redPots);
	assertEquals(null, hint.getBases());
	assertEquals(null, hint.getCovers());
	assertHintEqualsStoredHintsHtml(this, hint, testName+".html", REPRODUCE);
}


private static final String UNARY_X
="7.....4...2..7..8.4.3..8..9.7.5.631..6.12..9...1..7..6...3.29...3..4.16..49..1.35" + NL
+",89,68,26,13,59,,25,13,19,,56,49,,34,56,,13,,15,,26,15,,267,27,,89,,24,,89,,,,24,35,,45,,,34,78,,78,23,89,,48,39,,25,245,,16,15,78,,56,,,47,478,58,,2578,789,,59,,,28,268,,,78,68,,278,,";

/** Test UnaryChainHintX. */
@Test
public void testUnaryChainHintX() {
	final String testName = testName(this);
	getHints(UNARY_X, HINTER);

	// HINT 1
	AHint hint = find(HINTS, "Unary X Chain: G5+8");
////KEEP THESE: For use in creating test cases.
//System.out.format("aquaCells  : %s\n", diuf.sudoku.utils.Frmt.csv(hint.getAquaCells(0)));
//System.out.format("oranges    : %s\n", hint.getOranges(0));
//System.out.format("greens     : %s\n", hint.getGreens(0));
//System.out.format("reds       : %s\n", hint.getReds(0));
//System.out.format("redPots    : %s\n", hint.redPots);
//System.out.format("clueHtml   : %s\n", hint.getClueHtml(true));
//System.out.format("bases      : %s\n", diuf.sudoku.utils.Frmt.csv(hint.getBases()));
//System.out.format("covers     : %s\n", diuf.sudoku.utils.Frmt.csv(hint.getCovers()));
	checkAquaCells(hint, "G5");
	assertEquals(newPots("G5-8"), hint.getOranges(0));
	assertEquals(newPots("A4-8, B1-8, C7-8, E9-8, I5-8"), hint.getGreens(0));
	assertEquals(newPots("B6-8, C1-8, E4-8, G9-8, I7-8"), hint.getReds(0));
	assertEquals(newPots("G5-7"), hint.redPots);
	assertEquals(null, hint.getBases());
	assertEquals(null, hint.getCovers());
	assertHintEqualsStoredHintsHtml(this, hint, testName+"_1.html", REPRODUCE, false);

	// HINT 2
	hint = find(HINTS, "Unary X Chain: I5-8");
	assertHintEqualsStoredHintsHtml(this, hint, testName+"_2.html", REPRODUCE);
}


// ===========================================================
/** NB: There's no such thing as a Y Chain, only a Y Cycle. */
// ===========================================================


private static final String UNARY_XY_CYCLE
="..7..8.6.1...5...8...6.....7...6..35..4..7....9.2......5.4.3.2.47..2.3......16.4." + NL
+"259,234,,13,349,,2459,,1234,,2346,2369,37,,249,2479,79,,2389,2348,23589,,3479,124,12479,57,1234,,128,28,189,,149,12489,,,23568,12368,,359,38,,12689,189,1269,368,,3568,,348,145,14678,178,1467,689,,1689,,789,,16789,,1679,,,1689,89,,59,,158,16,2389,238,2389,57,,,5789,,79";
//   25	    412,027,925	26	 187	  1	Unary Chain                   	Bidirectional XY Cycle: D9, F8, H8, H3, E3, D2 (G3-7)

/** Test UnaryChain finds an XY Cycle. */
@Test
public void testUnaryXYCycle() {
	final String testName = testName(this);
	getHints(UNARY_XY_CYCLE, HINTER);

	AHint hint = find(HINTS, "Bidirectional XY Cycle: D9, F8, H8, H3, E3, D2");
////KEEP THESE: For use in creating test cases.
//System.out.format("aquaCells  : %s\n", diuf.sudoku.utils.Frmt.csv(hint.getAquaCells(0)));
//System.out.format("oranges    : %s\n", hint.getOranges(0));
//System.out.format("greens     : %s\n", hint.getGreens(0));
//System.out.format("reds       : %s\n", hint.getReds(0));
//System.out.format("redPots    : %s\n", hint.redPots);
//System.out.format("clueHtml   : %s\n", hint.getClueHtml(true));
//System.out.format("bases      : %s\n", diuf.sudoku.utils.Frmt.csv(hint.getBases()));
//System.out.format("covers     : %s\n", diuf.sudoku.utils.Frmt.csv(hint.getCovers()));
	checkAquaCells(hint, "D9, F8, H8, H3, E3, D2");
	assertEquals(newPots("D2-7, D9-5, H3-7, H8-5"), hint.getGreens(0));
	assertEquals(newPots("D9-7, E3-7, F8-5, G3-7, H3-5"), hint.getReds(0));
	assertEquals(newPots("G3-7"), hint.redPots);
	assertEquals(null, hint.getBases());
	assertEquals(null, hint.getCovers());
	assertEquals("Look for a Bidirectional XY Cycle at <b>D2</b>", hint.getClueHtml(true));
	assertHintEqualsStoredHintsHtml(this, hint, testName+".html", REPRODUCE);
}


private static final String UNARY_Y_CYCLE
=".....187..7..9.31..13..7..51..6.45...56.2.1492481596374.....261.6..1..5...1.467.3" + NL
+"59,29,45,35,36,,,,246,68,,245,458,,258,,,46,68,,,24,68,,49,29,,,39,79,,37,,,28,28,37,,,378,,38,,,,,,,,,,,,,,389,579,35789,378,35,,,,37,,27,789,,238,49,,48,59,28,,258,,,,89,";
// 61/10#top1465.d5.mt: 61	     14,218,948	42	  91	  1	Unary Chain                   	Bidirectional Y Cycle: A9, H9, B9, B1, A1 (D9-8)

/** Test UnaryChain finds a Y Cycle. */
@Test
public void testUnaryYCycle() {
	final String testName = testName(this);
	getHints(UNARY_Y_CYCLE, HINTER);

	AHint hint = find(HINTS, "Bidirectional Y Cycle: A9, H9, B9, B1, A1");
////KEEP THESE: For use in creating test cases.
//System.out.format("aquaCells  : %s\n", diuf.sudoku.utils.Frmt.csv(hint.getAquaCells(0)));
//System.out.format("oranges    : %s\n", hint.getOranges(0));
//System.out.format("greens     : %s\n", hint.getGreens(0));
//System.out.format("reds       : %s\n", hint.getReds(0));
//System.out.format("redPots    : %s\n", hint.redPots);
//System.out.format("clueHtml   : %s\n", hint.getClueHtml(true));
//System.out.format("bases      : %s\n", diuf.sudoku.utils.Frmt.csv(hint.getBases()));
//System.out.format("covers     : %s\n", diuf.sudoku.utils.Frmt.csv(hint.getCovers()));
	checkAquaCells(hint, "A9, H9, B9, B1, A1");
	assertEquals(newPots("A1-5, A9-9, B1-9, B9-2, H9-8"), hint.getGreens(0));
	assertEquals(newPots("A1-9, A9-5, B1-2, B9-8, D9-8, H9-9"), hint.getReds(0));
	assertEquals(newPots("D9-8"), hint.redPots);
	assertEquals(null, hint.getBases());
	assertEquals(null, hint.getCovers());
	assertEquals("Look for a Bidirectional Y Cycle at <b>A1</b>", hint.getClueHtml(true));
	assertHintEqualsStoredHintsHtml(this, hint, testName+".html", REPRODUCE);
}

}
