/*
 * Project: Sudoku Explainer
 * Copyright (C) 2006-2007 Nicolas Juillerat
 * Copyright (C) 2013-2020 Keith Corlett
 * Available under the terms of the Lesser General Public License (LGPL)
 */
package diuf.sudoku.solver.hinters.color;

import static diuf.sudoku.TestHelp.*;
import diuf.sudoku.solver.AHint;
import org.junit.Test;
import static org.junit.Assert.*;

/**
 * Test the Coloring hinter.
 * 
 * @author Keith Corlett 2020-09-23
 */
public class ColoringTest {

private static final boolean REPRODUCE = REPRODUCE_ALL_TESTS || false;

private static final Coloring HINTER = new Coloring();

// ============================================================================

private static final String SIMPLE_COLOR_TRAP
="...8...3...5...7.....1.......4.5.9..18.7.....35.4...72..7.12..6....745...4.....17"+NL
+"24679,2679,16,,2469,57,1246,,459,48,169,,29,3469,369,,24689,18,246789,23679,38,,2469,57,2468,245689,4589,267,267,,236,,13,,68,138,,,29,,269,369,346,456,45,,,69,,689,1689,16,,,59,39,,359,,,48,48,,268,12,18,36,,,,29,39,569,,236,59,3689,3689,23,,";
/*
12#C:\Users\User\Documents\NetBeansProjects\DiufSudoku\top1465.d5.mt
...8...3...5...7.....1.......4.5.9..18.7.....35.4...72..7.12..6....745...4.....17
24679,2679,16,,2469,57,1246,,459,48,169,,29,3469,369,,24689,18,246789,23679,38,,2469,57,2468,245689,4589,267,267,,236,,13,,68,138,,,29,,269,369,346,456,45,,,69,,689,1689,16,,,59,39,,359,,,48,48,,268,12,18,36,,,,29,39,569,,236,59,3689,3689,23,,
64   	      1,508,900	26	 162	  1	Coloring                      	Simple Color Trap: C9-2, D2-2, E5-2, H8-2, C5-2, D4-2, G9-2, H2-2 on 2 (H3-2)
*/
/**
 * Coloring.findHints a Simple Color Trap.
 */
@Test
public void test1() {
	final String testName = testName(this);
	getHints(SIMPLE_COLOR_TRAP, HINTER);
	
	// find my hint
	AHint hint = find(HINTS, "Simple Color Trap: C9-2, D2-2, E5-2, H8-2, C5-2, D4-2, G9-2, H2-2 on 2");
////KEEP THESE: For use in creating test cases.
//System.out.format("aquaCells : %s\n", diuf.sudoku.utils.Frmt.csv(hint.getAquaCells(0)));
//System.out.format("blues     : %s\n", hint.getBlues(grid, 0));
//System.out.format("greens    : %s\n", hint.getGreens(0));
//System.out.format("oranges   : %s\n", hint.getOranges(0));
//System.out.format("reds      : %s\n", hint.getReds(0));
//System.out.format("redPots   : %s\n", hint.redPots);
//System.out.format("clueHtml  : %s\n", hint.getClueHtml(true));
//System.out.format("bases     : %s\n", diuf.sudoku.utils.Frmt.csv(hint.getBases()));
//System.out.format("covers    : %s\n", diuf.sudoku.utils.Frmt.csv(hint.getCovers()));
	checkClue(hint, "Look for a Simple Color Trap on <b>2</b>");
	assertEquals(grid.getCSVSet("C9, D2, E5, H8, C5, D4, G9, H2"), hint.getAquaCells(0));
	assertEquals(newPots("C5-2, D4-2, G9-2, H2-2"), hint.getBlues(grid, 0));
	assertEquals(newPots("C9-2, D2-2, E5-2, H8-2"), hint.getGreens(0));
	assertEquals(null, hint.getOranges(0));
	assertEquals(newPots("H3-2"), hint.getReds(0));
	assertEquals(newPots("H3-2"), hint.redPots);
	assertEquals(null, hint.getBases());
	assertEquals(null, hint.getCovers());
	assertHintEqualsStoredHintsHtml(this, hint, testName+".html", REPRODUCE);
}

// ============================================================================

private static final String SIMPLE_COLOR_WRAP
=".475...311....7.4595.4.16875317.84.2.9.142753274.5381.41587.32.38.2.51747..3145.8"+NL
+"68,,,,28,69,29,,,,26,38,69,38,,29,,,,,23,,23,,,,,,,,,69,,,69,,68,,68,,,,,,,,,,69,,,,,69,,,,,,69,,,69,,,69,,69,,,,,,26,269,,,,,69,";
/*
665#C:\Users\User\Documents\NetBeansProjects\DiufSudoku\top1465.d5.mt
.475...311....7.4595.4.16875317.84.2.9.142753274.5381.41587.32.38.2.51747..3145.8
68,,,,28,69,29,,,,26,38,69,38,,29,,,,,23,,23,,,,,,,,,69,,,69,,68,,68,,,,,,,,,,69,,,,,69,,,,,,69,,,69,,,69,,69,,,,,,26,269,,,,,69,
54   	      1,300,900	57	  40	  9	Coloring                      	Simple Color Wrap: A1-6, B9-6, C5-6, C8-6, D2-6, E4-6, F7-6, H9-6, I6-6, A5-6, B2-6, D6-6, E8-6, F1-6, H4-6, I7-6 on 6 (A1-6, B9-6, C5-6, C8-6, D2-6, E4-6, F7-6, H9-6, I6-6)
*/
/**
 * Coloring.findHints a Simple Color Wrap.
 */
@Test
public void test2() {
	final String testName = testName(this);
	getHints(SIMPLE_COLOR_WRAP, HINTER);
	
	// find my hint
	// 1766082166	Simple Color Wrap: A5-6, B2-6, D6-6, E8-6, F1-6, H4-6, I7-6 on 6 (A1-6, B9-6, C5-6, C8-6, D2-6, E4-6, F7-6, H9-6, I6-6)
	AHint hint = find(HINTS, "Simple Color Wrap: A5-6, B2-6, D6-6, E8-6, F1-6, H4-6, I7-6 on 6");
////KEEP THESE: For use in creating test cases.
//System.out.format("aquaCells : %s\n", diuf.sudoku.utils.Frmt.csv(hint.getAquaCells(0)));
//System.out.format("blues     : %s\n", hint.getBlues(grid, 0));
//System.out.format("greens    : %s\n", hint.getGreens(0));
//System.out.format("oranges   : %s\n", hint.getOranges(0));
//System.out.format("reds      : %s\n", hint.getReds(0));
//System.out.format("redPots   : %s\n", hint.redPots);
//System.out.format("clueHtml  : %s\n", hint.getClueHtml(true));
//System.out.format("bases     : %s\n", diuf.sudoku.utils.Frmt.csv(hint.getBases()));
//System.out.format("covers    : %s\n", diuf.sudoku.utils.Frmt.csv(hint.getCovers()));
	checkClue(hint, "Look for a Simple Color Wrap on <b>6</b>");
	assertEquals(grid.getCSVSet("A5, B2, D6, E8, F1, H4, I7"), hint.getAquaCells(0));
	assertEquals(newPots("A5-6, B2-6, D6-6, E8-6, F1-6, H4-6, I7-6"), hint.getBlues(grid, 0));
	assertEquals(newPots(""), hint.getGreens(0));
	assertEquals(null, hint.getOranges(0));
	assertEquals(newPots("A1-6, B9-6, C5-6, C8-6, D2-6, E4-6, F7-6, H9-6, I6-6"), hint.getReds(0));
	assertEquals(newPots("A1-6, B9-6, C5-6, C8-6, D2-6, E4-6, F7-6, H9-6, I6-6"), hint.redPots);
	assertEquals(null, hint.getBases());
	assertEquals(null, hint.getCovers());
	assertHintEqualsStoredHintsHtml(this, hint, testName+".html", REPRODUCE);
}

// ============================================================================

//private static final String MULTI_COLOR_1
//="?"+NL
//+"?";

/**
 * Coloring.findHints a Multi Color 1.
 */
@Test
public void test3() {
	final String testName = testName(this);
//	getHints(MULTI_COLOR_1, HINTER);

// I don't produce a Multi Color 1 hint, indicating that either my code is
// broken (likely) or that there simply are no instance of Multi Color 1 in the
// whole of top1465.d5.mt, which seems unlikely to me; so there's officially an
// A-grade bug in Coloring which I have asbolutely no idea how to tackle, and
// therefore I have no plan to fix it; which is a big ole lazy/scaredy cop-out,
// and I therefore just need shootin! So please just go ahead and shoot me!
	System.out.println("WARNING: This test really should fail because I have no \"Multi Color 1\" to verify.");

}

// ============================================================================

private static final String MULTI_COLOR_2
=".4.3..9..973..5.2.8........6.9..4352..5....9...4...6.....9...683...42.79.9.16...."+NL
+"125,,126,,278,1678,,18,567,,,,46,18,,48,,146,,1256,126,2467,1279,1679,457,34,3567,,18,,78,178,,,,,127,1238,,268,123,1368,478,,147,127,1238,,258,2359,189,,18,17,124,125,127,,357,37,1245,,,,56,168,58,,,15,,,245,,78,,,78,245,34,35";
/*
13#C:\Users\User\Documents\NetBeansProjects\DiufSudoku\top1465.d5.mt
.4.3..9..973..5.2.8........6.9..4352..5....9...4...6.....9...683...42.79.9.16....
125,,126,,278,1678,,18,567,,,,46,18,,48,,146,,1256,126,2467,1279,1679,457,34,3567,,18,,78,178,,,,,127,1238,,268,123,1368,478,,147,127,1238,,258,2359,189,,18,17,124,125,127,,357,37,1245,,,,56,168,58,,,15,,,245,,78,,,78,245,34,35
37   	        696,700	30	 145	  2	Coloring                      	Multi Color 2: E2-1, H1-1, H6-1, I2-1, B4-1, E4-1 on 1 (A6-1, B6-1)
*/

/**
 * Coloring.findHints a Multi Color 2.
 */
@Test
public void test4() {
	final String testName = testName(this);
	getHints(MULTI_COLOR_2, HINTER);
	
	// find my hint
	AHint hint = find(HINTS, "Multi Color 2: E2-1, H1-1, H6-1, I2-1, B4-1, E4-1 on 1");
////KEEP THESE: For use in creating test cases.
//System.out.format("aquaCells : %s\n", diuf.sudoku.utils.Frmt.csv(hint.getAquaCells(0)));
//System.out.format("blues     : %s\n", hint.getBlues(grid, 0));
//System.out.format("greens    : %s\n", hint.getGreens(0));
//System.out.format("oranges   : %s\n", hint.getOranges(0));
//System.out.format("reds      : %s\n", hint.getReds(0));
//System.out.format("redPots   : %s\n", hint.redPots);
//System.out.format("clueHtml  : %s\n", hint.getClueHtml(true));
//System.out.format("bases     : %s\n", diuf.sudoku.utils.Frmt.csv(hint.getBases()));
//System.out.format("covers    : %s\n", diuf.sudoku.utils.Frmt.csv(hint.getCovers()));
	checkClue(hint, "Look for a Multi Color 2 on <b>1</b>");
	assertEquals(grid.getCSVSet("I2, B4, E2, E4, H1, H6"), hint.getAquaCells(0));
	assertEquals(newPots("H6-1, I2-1"), hint.getBlues(grid, 0));
	assertEquals(newPots("E2-1, H1-1"), hint.getGreens(0));
	assertEquals(null, hint.getOranges(0));
	assertEquals(newPots("A6-1, B6-1"), hint.getReds(0));
	assertEquals(newPots("A6-1, B6-1"), hint.redPots);
	assertEquals(null, hint.getBases());
	assertEquals(null, hint.getCovers());
	assertHintEqualsStoredHintsHtml(this, hint, testName+".html", REPRODUCE);
}

}
