/*
 * Project: Sudoku Explainer
 * Copyright (C) 2006-2007 Nicolas Juillerat
 * Copyright (C) 2013-2020 Keith Corlett
 * Available under the terms of the Lesser General Public License (LGPL)
 */
package diuf.sudoku.solver.hinters.hdnset;

import static org.junit.Assert.*;
import static diuf.sudoku.TestHelp.*;
import org.junit.Test;
import diuf.sudoku.Tech;
import diuf.sudoku.solver.AHint;
import diuf.sudoku.solver.hinters.AHinter;


/**
 * Test the HiddenSet hinter.
 * @author Keith Corlett 2018 Apr
 */
public final class HiddenSetTest {

private static final boolean REPRODUCE = REPRODUCE_ALL_TESTS || false;

private static final String DIRECT_HIDDEN_PAIR
="...6....4..53.9..1....8.73..1...6...6.3..8...7..91....2..........7...58..9....6.3";
//8#C:\Users\User\Documents\NetBeansProjects\DiufSudoku\top1465.d3.mt

/** Direct Hidden Pair */
@Test
public void testDirectHiddenPair() {
	final String testName = testName(this);
	AHinter hinter = new HiddenSet(Tech.DirectHiddenPair); // NOTE DIRECT!
	getHints(DIRECT_HIDDEN_PAIR, hinter);
	//System.out.println(accu.getHint().toString());
	AHint hint = find(HINTS, "Direct Hidden Pair: E7, E8: 6, 9 in col E", "HiddenSetDirectHint");
//// Keep these, it's how I create a test case. I'm only resting for repeated.
//System.out.format("SimpleName : %s\n", hint.getClass().getSimpleName());
//System.out.format("cell       : %s\n", hint.cell);
//System.out.format("value      : %s\n", hint.value);
//System.out.format("ClueHtml   : %s\n", hint.getClueHtml(true));
//System.out.format("AquaCells  : %s\n", diuf.sudoku.utils.Frmt.csv(hint.getAquaCells(0)));
//System.out.format("Oranges    : %s\n", hint.getOranges(0));
//System.out.format("Greens     : %s\n", hint.getGreens(0));
//System.out.format("Reds       : %s\n", hint.getReds(0));
//System.out.format("RedPots    : %s\n", hint.redPots);
//System.out.format("bases      : %s\n", diuf.sudoku.utils.Frmt.csv(hint.getBases()));
//System.out.format("covers     : %s\n", diuf.sudoku.utils.Frmt.csv(hint.getCovers()));
	assertEquals(grid.get("E4"), hint.cell);
	assertEquals(3, hint.value);
	checkClue(hint, "Look for a Direct Hidden Pair in <b1>col E</b1>");
	assertEquals(grid.getCSVSet("E4"), hint.getAquaCells(0));
	assertEquals(newPots("E7-69, E8-69"), hint.getOranges(0));
	assertEquals(newPots("E4-3"), hint.getGreens(0));
	assertEquals(newPots("E7-3457, E8-234"), hint.getReds(0));
	assertEquals(newPots("E7-3457, E8-234"), hint.getReds(0));
	assertEquals(grid.cols[4], hint.getBases().get(0));
	assertEquals(null, hint.getCovers());
	assertHintEqualsStoredHintsHtml(this, hint, testName+".html", REPRODUCE);
}


private static final String DIRECT_HIDDEN_TRIPLE
="..1...6..96......73...6.89.8....93.5.3.4..2..2..835...6..1.2..4.2..4.......3.8526" + NL
+"47,578,,579,2589,347,,345,23,,,28,25,28,134,14,1345,,,57,2457,257,,14,,,12,,147,67,267,12,,,1467,,15,,59,,17,67,,678,89,,1479,679,,,,147,1467,19,,589,38,,59,,79,378,,57,,3589,569,,67,19,138,38,14,19,47,,79,,,,";
/*
..1...6..96......73...6.89.8....93.5.3.4..2..2..835...6..1.2..4.2..4.......3.8526
47,578,,579,2589,347,,345,23,,,28,25,28,134,14,1345,,,57,2457,257,,14,,,12,,147,67,267,12,,,1467,,15,,59,,17,67,,678,89,,1479,679,,,,147,1467,19,,589,38,,59,,79,378,,57,,3589,569,,67,19,138,38,14,19,47,,79,,,,
2#C:\Users\User\Documents\NetBeansProjects\DiufSudoku\top1465.d3.mt
*/

/** Hidden Pair */
@Test
public void testHiddenPair() {
	final String testName = testName(this);
	AHinter hinter = new HiddenSet(Tech.HiddenPair);
	getHints(DIRECT_HIDDEN_PAIR, hinter);

	// HINT 1
	//System.out.println(accu.getHint().toString());
	AHint hint = find(HINTS, "Hidden Pair: E7, E8: 6, 9 in box 8");
//// Keep these, it's how I create a test case. I'm only resting for repeated.
//System.out.format("SimpleName : %s\n", hint.getClass().getSimpleName());
//System.out.format("cell       : %s\n", hint.cell);
//System.out.format("value      : %s\n", hint.value);
//System.out.format("ClueHtml   : %s\n", hint.getClueHtml(true));
//System.out.format("AquaCells  : %s\n", diuf.sudoku.utils.Frmt.csv(hint.getAquaCells(0)));
//System.out.format("Oranges    : %s\n", hint.getOranges(0));
//System.out.format("Greens     : %s\n", hint.getGreens(0));
//System.out.format("Reds       : %s\n", hint.getReds(0));
//System.out.format("RedPots    : %s\n", hint.redPots);
//System.out.format("bases      : %s\n", diuf.sudoku.utils.Frmt.csv(hint.getBases()));
//System.out.format("covers     : %s\n", diuf.sudoku.utils.Frmt.csv(hint.getCovers()));
	assertEquals(false, hint.getClass().getSimpleName().contains("Direct"));
	assertEquals(null, hint.cell);
	assertEquals(0, hint.value);
	checkClue(hint, "Look for a Hidden Pair in <b1>box 8</b1>");
	assertEquals(grid.getCSVSet("E7, E8"), hint.getAquaCells(0));
	assertEquals(null, hint.getOranges(0));
	assertEquals(newPots("E7-69, E8-69"), hint.getGreens(0));
	assertEquals(newPots("E7-3457, E8-234"), hint.getReds(0));
	assertEquals(newPots("E7-3457, E8-234"), hint.redPots);
	assertEquals(grid.boxs[7], hint.getBases().get(0));
	assertEquals(null, hint.getCovers());
	assertHintEqualsStoredHintsHtml(this, hint, testName+"_1.html", REPRODUCE, false);

	// HINT 2
	//System.out.println(accu.getHint().toString());
	hint = find(HINTS, "Hidden Pair: E7, E8: 6, 9 in col E");
//// Keep these, it's how I create a test case. I'm only resting for repeated.
//System.out.format("SimpleName : %s\n", hint.getClass().getSimpleName());
//System.out.format("cell       : %s\n", hint.cell);
//System.out.format("value      : %s\n", hint.value);
//System.out.format("ClueHtml   : %s\n", hint.getClueHtml(true));
//System.out.format("AquaCells  : %s\n", diuf.sudoku.utils.Frmt.csv(hint.getAquaCells(0)));
//System.out.format("Oranges    : %s\n", hint.getOranges(0));
//System.out.format("Greens     : %s\n", hint.getGreens(0));
//System.out.format("Reds       : %s\n", hint.getReds(0));
//System.out.format("RedPots    : %s\n", hint.redPots);
//System.out.format("bases      : %s\n", diuf.sudoku.utils.Frmt.csv(hint.getBases()));
//System.out.format("covers     : %s\n", diuf.sudoku.utils.Frmt.csv(hint.getCovers()));
	assertEquals(false, hint.getClass().getSimpleName().contains("Direct"));
	assertEquals(null, hint.cell);
	assertEquals(0, hint.value);
	checkClue(hint, "Look for a Hidden Pair in <b1>col E</b1>");
	assertEquals(grid.getCSVSet("E7, E8"), hint.getAquaCells(0));
	assertEquals(null, hint.getOranges(0));
	assertEquals(newPots("E7-69, E8-69"), hint.getGreens(0));
	assertEquals(newPots("E7-3457, E8-234"), hint.getReds(0));
	assertEquals(newPots("E7-3457, E8-234"), hint.redPots);
	assertEquals(grid.cols[4], hint.getBases().get(0));
	assertEquals(null, hint.getCovers());
	assertHintEqualsStoredHintsHtml(this, hint, testName+"_2.html", REPRODUCE);
}

/** Direct Hidden Triple */
@Test
public void testDirectHiddenTriple() {
	final String testName = testName(this);
	AHinter hinter = new HiddenSet(Tech.DirectHiddenTriple);
	getHints(DIRECT_HIDDEN_TRIPLE, hinter);
	//System.out.println(accu.getHint().toHtml());
	AHint hint = find(HINTS, "Direct Hidden Triple: F2, G2, H2: 1, 3, 4 in row 2");
//// Keep these, it's how I create a test case. I'm only resting for repeated.
//System.out.format("SimpleName : %s\n", hint.getClass().getSimpleName());
//System.out.format("cell       : %s\n", hint.cell);
//System.out.format("value      : %s\n", hint.value);
//System.out.format("ClueHtml   : %s\n", hint.getClueHtml(true));
//System.out.format("AquaCells  : %s\n", diuf.sudoku.utils.Frmt.csv(hint.getAquaCells(0)));
//System.out.format("Oranges    : %s\n", hint.getOranges(0));
//System.out.format("Greens     : %s\n", hint.getGreens(0));
//System.out.format("Reds       : %s\n", hint.getReds(0));
//System.out.format("RedPots    : %s\n", hint.redPots);
//System.out.format("bases      : %s\n", diuf.sudoku.utils.Frmt.csv(hint.getBases()));
//System.out.format("covers     : %s\n", diuf.sudoku.utils.Frmt.csv(hint.getCovers()));
	assertEquals(true, hint.getClass().getSimpleName().contains("Direct"));
	assertEquals(grid.get("D2"), hint.cell);
	assertEquals(5, hint.value);
	checkClue(hint, "Look for a Direct Hidden Triple in <b1>row 2</b1>");
	assertEquals(grid.getCSVSet("D2"), hint.getAquaCells(0));
	assertEquals(newPots("F2-134, G2-14, H2-134"), hint.getOranges(0));
	assertEquals(newPots("D2-5"), hint.getGreens(0));
	assertEquals(newPots("H2-5"), hint.getReds(0));
	assertEquals(newPots("H2-5"), hint.redPots);
	assertEquals(grid.rows[1], hint.getBases().get(0));
	assertEquals(null, hint.getCovers());
	assertHintEqualsStoredHintsHtml(this, hint, testName+".html", REPRODUCE);
}

/** Hidden Triple */
@Test
public void testHiddenTriple() {
	final String testName = testName(this);
	AHinter hinter = new HiddenSet(Tech.HiddenTriple); // INDIRECT
	getHints(DIRECT_HIDDEN_TRIPLE, hinter);

	// HINT 1
	//System.out.println(accu.getHint().toHtml());
	AHint hint = find(HINTS, "Hidden Triple: F1, F2, F3: 1, 3, 4 in box 2");
//// Keep these, it's how I create a test case. I'm only resting for repeated.
//System.out.format("SimpleName : %s\n", hint.getClass().getSimpleName());
//System.out.format("cell       : %s\n", hint.cell);
//System.out.format("value      : %s\n", hint.value);
//System.out.format("ClueHtml   : %s\n", hint.getClueHtml(true));
//System.out.format("AquaCells  : %s\n", diuf.sudoku.utils.Frmt.csv(hint.getAquaCells(0)));
//System.out.format("Oranges    : %s\n", hint.getOranges(0));
//System.out.format("Greens     : %s\n", hint.getGreens(0));
//System.out.format("Reds       : %s\n", hint.getReds(0));
//System.out.format("RedPots    : %s\n", hint.redPots);
//System.out.format("bases      : %s\n", diuf.sudoku.utils.Frmt.csv(hint.getBases()));
//System.out.format("covers     : %s\n", diuf.sudoku.utils.Frmt.csv(hint.getCovers()));
	assertEquals(false, hint.getClass().getSimpleName().contains("Direct"));
	assertEquals(null, hint.cell);
	assertEquals(0, hint.value);
	checkClue(hint, "Look for a Hidden Triple in <b1>box 2</b1>");
	assertEquals(grid.getCSVSet("F1, F2, F3"), hint.getAquaCells(0));
	assertEquals(null, hint.getOranges(0));
	assertEquals(newPots("F1-34, F2-134, F3-14"), hint.getGreens(0));
	assertEquals(newPots("F1-7"), hint.getReds(0));
	assertEquals(newPots("F1-7"), hint.redPots);
	assertEquals(grid.boxs[1], hint.getBases().get(0));
	assertEquals(null, hint.getCovers());
	assertHintEqualsStoredHintsHtml(this, hint, testName+"_1.html", REPRODUCE, false);

	// HINT 2
	//System.out.println(accu.getHint().toHtml());
	hint = find(HINTS, "Hidden Triple: F2, G2, H2: 1, 3, 4 in row 2", "HiddenSetHint");
//// Keep these, it's how I create a test case. I'm only resting for repeated.
//System.out.format("SimpleName : %s\n", hint.getClass().getSimpleName());
//System.out.format("cell       : %s\n", hint.cell);
//System.out.format("value      : %s\n", hint.value);
//System.out.format("ClueHtml   : %s\n", hint.getClueHtml(true));
//System.out.format("AquaCells  : %s\n", diuf.sudoku.utils.Frmt.csv(hint.getAquaCells(0)));
//System.out.format("Oranges    : %s\n", hint.getOranges(0));
//System.out.format("Greens     : %s\n", hint.getGreens(0));
//System.out.format("Reds       : %s\n", hint.getReds(0));
//System.out.format("RedPots    : %s\n", hint.redPots);
//System.out.format("bases      : %s\n", diuf.sudoku.utils.Frmt.csv(hint.getBases()));
//System.out.format("covers     : %s\n", diuf.sudoku.utils.Frmt.csv(hint.getCovers()));
	assertEquals(false, hint.getClass().getSimpleName().contains("Direct"));
	assertEquals(null, hint.cell);
	assertEquals(0, hint.value);
	checkClue(hint, "Look for a Hidden Triple in <b1>row 2</b1>");
	assertEquals(grid.getCSVSet("F2, G2, H2"), hint.getAquaCells(0));
	assertEquals(newPots("F2-134 G2-14 H2-134"), hint.getGreens(0));
	assertEquals(newPots("H2-5"), hint.getReds(0));
	assertEquals(newPots("H2-5"), hint.redPots);
	assertEquals(grid.rows[1], hint.getBases().get(0));
	assertEquals(null, hint.getCovers());
	assertHintEqualsStoredHintsHtml(this, hint, testName+"_2.html", REPRODUCE, false);

	// HINT 3
	//System.out.println(accu.getHint().toHtml());
	hint = find(HINTS, "Hidden Triple: F1, F2, F3: 1, 3, 4 in col F", "HiddenSetHint");
//// Keep these, it's how I create a test case. I'm only resting for repeated.
//System.out.format("SimpleName : %s\n", hint.getClass().getSimpleName());
//System.out.format("cell       : %s\n", hint.cell);
//System.out.format("value      : %s\n", hint.value);
//System.out.format("ClueHtml   : %s\n", hint.getClueHtml(true));
//System.out.format("AquaCells  : %s\n", diuf.sudoku.utils.Frmt.csv(hint.getAquaCells(0)));
//System.out.format("Oranges    : %s\n", hint.getOranges(0));
//System.out.format("Greens     : %s\n", hint.getGreens(0));
//System.out.format("Reds       : %s\n", hint.getReds(0));
//System.out.format("RedPots    : %s\n", hint.redPots);
//System.out.format("bases      : %s\n", diuf.sudoku.utils.Frmt.csv(hint.getBases()));
//System.out.format("covers     : %s\n", diuf.sudoku.utils.Frmt.csv(hint.getCovers()));
	assertEquals(false, hint.getClass().getSimpleName().contains("Direct"));
	assertEquals(null, hint.cell);
	assertEquals(0, hint.value);
	checkClue(hint, "Look for a Hidden Triple in <b1>col F</b1>");
	assertEquals(grid.getCSVSet("F1, F2, F3"), hint.getAquaCells(0));
	assertEquals(newPots("F1-34 F2-134 F3-14"), hint.getGreens(0));
	assertEquals(newPots("F1-7"), hint.getReds(0));
	assertEquals(newPots("F1-7"), hint.redPots);
	assertEquals(grid.cols[5], hint.getBases().get(0));
	assertEquals(null, hint.getCovers());
	assertHintEqualsStoredHintsHtml(this, hint, testName+"_3.html", REPRODUCE);
}


private static final String HIDDEN_QUAD
=".29..........4.69.7461593....8.1573..........172.835.92.4..69.1...59.8..9........" + NL
+"358,,,368,367,78,14,14,57,358,135,135,237,,278,,,57,,,,,,,,28,28,46,69,,249,,,,,24,345,359,35,479,26,47,12,1468,468,,,,46,,,,46,,,358,,378,37,,,57,,36,136,137,,,1247,,267,2346,,13568,1357,3478,237,178,24,4567,346";
//142#C:\Users\User\Documents\NetBeansProjects\DiufSudoku\top1465.d3.mt

/** Hidden Quad */
@Test
public void testHiddenQuad() {
	final String testName = testName(this);
	AHinter hinter = new HiddenSet(Tech.HiddenQuad); // INDIRECT
	getHints(HIDDEN_QUAD, hinter);
	//System.out.println(accu.getHint().toString());
	AHint hint = find(HINTS, "Hidden Quad: E5, G5, H5, I5: 1, 2, 6, 8 in row 5");
//// Keep these, it's how I create a test case. I'm only resting for repeated.
//System.out.format("SimpleName : %s\n", hint.getClass().getSimpleName());
//System.out.format("cell       : %s\n", hint.cell);
//System.out.format("value      : %s\n", hint.value);
//System.out.format("ClueHtml   : %s\n", hint.getClueHtml(true));
//System.out.format("AquaCells  : %s\n", diuf.sudoku.utils.Frmt.csv(hint.getAquaCells(0)));
//System.out.format("Oranges    : %s\n", hint.getOranges(0));
//System.out.format("Greens     : %s\n", hint.getGreens(0));
//System.out.format("Reds       : %s\n", hint.getReds(0));
//System.out.format("RedPots    : %s\n", hint.redPots);
//System.out.format("bases      : %s\n", diuf.sudoku.utils.Frmt.csv(hint.getBases()));
//System.out.format("covers     : %s\n", diuf.sudoku.utils.Frmt.csv(hint.getCovers()));
	assertEquals(false, hint.getClass().getSimpleName().contains("Direct"));
	assertEquals(null, hint.cell);
	assertEquals(0, hint.value);
	checkClue(hint, "Look for a Hidden Quad in <b1>row 5</b1>");
	assertEquals(grid.getCSVSet("E5, G5, H5, I5"), hint.getAquaCells(0));
	assertEquals(newPots("E5-26, G5-12, H5-168, I5-68"), hint.getGreens(0));
	assertEquals(newPots("H5-4, I5-4"), hint.getReds(0));
	assertEquals(newPots("H5-4, I5-4"), hint.redPots); // (H5-4, I5-4)
	assertEquals(grid.rows[4], hint.getBases().get(0));
	assertEquals(null, hint.getCovers());
	assertHintEqualsStoredHintsHtml(this, hint, testName+".html", REPRODUCE);
}

}