/*
 * Project: Sudoku Explainer
 * Copyright (C) 2006-2007 Nicolas Juillerat
 * Copyright (C) 2013-2020 Keith Corlett
 * Available under the terms of the Lesser General Public License (LGPL)
 */
package diuf.sudoku.solver.hinters.lock2;

import static org.junit.Assert.*;
import static diuf.sudoku.TestHelp.*;
import diuf.sudoku.solver.AHint;
import diuf.sudoku.solver.hinters.AHinter;
import org.junit.Test;

/**
 * Test the Locking hinter.
 * @author Keith Corlett 2018 Apr
 */
public final class LockingGeneralisedTest {
	
private static final boolean REPRODUCE = REPRODUCE_ALL_TESTS || false;

	
private static final String PUZZLE
="7.....4...2..7..8...3..8..9...5..3...6..2..9...1..7..6...3..9...3..4..6...9..1.35" + NL
+",1589,568,1269,13569,23569,,125,123,14569,,456,1469,,34569,156,,13,1456,145,,1246,156,,12567,1257,,2489,4789,2478,,1689,469,,1247,12478,3458,,4578,148,,34,1578,,1478,234589,4589,,489,389,,258,245,,124568,14578,245678,,568,256,,1247,12478,1258,,2578,2789,,259,1278,,1278,2468,478,,2678,68,,278,,";
//1#C:\Users\User\Documents\SodukuPuzzles\MagicTour\top1465.d3.mt;
//Pointing: box 9 and row 7 on 4 (A7-4, B7-4, C7-4)
//Claiming: row 9 and box 7 on 4 (A7-4, B7-4, C7-4)

/** Test of getHints method, of class LockingGeneralised. */
@Test
public void test1() {
	final String testName = testName(this);
	// then create the hinter
	final AHinter HINTER = new LockingGeneralised();
	getHints(PUZZLE, HINTER);

	// POINTING HINT
	AHint hint = find(HINTS, "Locking Generalised: H7 I7 on 4 in box 9 (A7-4, B7-4, C7-4)");
////KEEP THESE: For use in creating test cases.
//System.out.format("hint        : %s\n", hint.toFullString());
//System.out.format("clueHtml    : %s\n", hint.getClueHtml(true));
//System.out.format("aquaCells   : %s\n", diuf.sudoku.utils.Frmt.csv(hint.getAquaCells(0)));
//System.out.format("oranges     : %s\n", hint.getOranges(0));
//System.out.format("greens      : %s\n", hint.getGreens(0));
//System.out.format("reds        : %s\n", hint.getReds(0));
//System.out.format("bases       : %s\n", diuf.sudoku.utils.Frmt.csv(hint.getBases()));
	checkClue(hint, "Look for a Locking Generalised in <b1>box 9</b1>");
	checkAquaCells(hint, "I7, H7");
	assertEquals(newPots(4, "H7","I7"), hint.getGreens(0));
	assertEquals(newPots(4, "A7","B7","C7"), hint.getReds(0));
	assertEquals(newPots(4, "A7","B7","C7"), hint.redPots);
	assertEquals(grid.boxs[8], hint.getBases().get(0));
//there are no covers in a LockingGenHint, mainly because I have no idea HOW
//to put it in there, ie how to determine which region the elimination/s share
//with all of the given cells.
//	assertEquals(grid.rows[6], hint.getCovers().get(0));
	assertHintEqualsStoredHintsHtml(this, hint, testName+".html", REPRODUCE);
}

}