/*
 * Project: Sudoku Explainer
 * Copyright (C) 2006-2007 Nicolas Juillerat
 * Copyright (C) 2013-2020 Keith Corlett
 * Available under the terms of the Lesser General Public License (LGPL)
 */
package diuf.sudoku.solver.hinters.sdp;

import static diuf.sudoku.TestHelp.*;
import diuf.sudoku.solver.AHint;
import diuf.sudoku.solver.hinters.AHinter;
import diuf.sudoku.utils.Frmt;
import org.junit.Test;
import static org.junit.Assert.*;


/**
 * Test the EmptyRectangle hinter.
 *
 * @author Keith Corlett 2021-02-10
 */
public class EmptyRectangleTest {

private static final boolean REPRODUCE = REPRODUCE_ALL_TESTS || false;

/*
1#C:\Users\User\Documents\NetBeansProjects\DiufSudoku\top1465.d5.mt
7.....4...2..7..8...3..8..9...5.63...6..2..9...1..7..6...3..9...3..4..6...9..1.35
,189,568,1269,1369,2359,,125,123,19,,56,149,,345,156,,13,146,145,,126,156,,1267,1257,,289,789,247,,189,,,1247,12478,35,,457,18,,34,1578,,1478,2358,589,,489,39,,258,245,,12568,1578,25678,,568,25,,1247,12478,1258,,2578,2789,,259,1278,,1278,2468,478,,2678,68,,278,,
26   	      1,142,500	26	 181	  1	Empty Rectangle               	Empty Rectangle: E3, E7 on 5 (C7-5)
*/
private static final String PUZZLE
="7.....4...2..7..8...3..8..9...5.63...6..2..9...1..7..6...3..9...3..4..6...9..1.35" + NL
+",189,568,1269,1369,2359,,125,123,19,,56,149,,345,156,,13,146,145,,126,156,,1267,1257,,289,789,247,,189,,,1247,12478,35,,457,18,,34,1578,,1478,2358,589,,489,39,,258,245,,12568,1578,25678,,568,25,,1247,12478,1258,,2578,2789,,259,1278,,1278,2468,478,,2678,68,,278,,";

/** Test EmptyRectangle */
@Test
public void test() {
	final String testName = testName(this);
	AHinter hinter = new EmptyRectangle();
	getHints(PUZZLE, hinter);
	AHint hint = find(HINTS, "Empty Rectangle: E3, E7 on 5");
if(REPRODUCE) {
//KEEP THESE: to maintain this test case.
System.out.format("clueHtml : %s\n", hint.getClueHtml(true));
System.out.format("aquaCells: %s\n", Frmt.csv(hint.getAquaCells(0)));
System.out.format("oranges  : %s\n", hint.getOranges(0));
System.out.format("reds     : %s\n", hint.getReds(0));
System.out.format("bases    : %s\n", diuf.sudoku.utils.Frmt.csv(hint.getBases()));
System.out.format("covers   : %s\n", diuf.sudoku.utils.Frmt.csv(hint.getCovers()));
}
	checkClue(hint, "Look for a Empty Rectangle on 5");
	assertEquals(grid.getCSVSet("E3, E7"), hint.getAquaCells(0));
	assertEquals(newPots("E3-5, E7-5"), hint.getOranges(0));
	assertEquals(newPots("C7-5"), hint.getReds(0));
	assertArrayEquals(grid.regions("box 1"), hint.getBasesArray());
	assertArrayEquals(grid.regions("box 1, col C, col E, row 7"), hint.getCoversArray());
	assertHintEqualsStoredHintsHtml(this, hint, testName+".html", REPRODUCE);
}

}
