/*
 * Project: Sudoku Explainer
 * Copyright (C) 2006-2007 Nicolas Juillerat
 * Copyright (C) 2013-2020 Keith Corlett
 * Available under the terms of the Lesser General Public License (LGPL)
 */
package diuf.sudoku.solver.hinters.sdp;

import static diuf.sudoku.TestHelp.*;
import diuf.sudoku.solver.AHint;
import diuf.sudoku.solver.hinters.AHinter;
import diuf.sudoku.utils.Frmt;
import org.junit.Test;
import static org.junit.Assert.*;


/**
 * Test the Skyscraper hinter.
 *
 * @author Keith Corlett 2021-02-10
 */
public class SkyscraperTest {

private static final boolean REPRODUCE = REPRODUCE_ALL_TESTS || false;

/*
39#C:\Users\User\Documents\NetBeansProjects\DiufSudoku\top1465.d5.mt
.5..4.8...6.5....9...............1..9..231..73.1..7..2..4.9.....376....5.987...2.
17,,239,139,,2369,,1367,136,48,,23,,1278,238,2347,1347,,48,17,239,189,12678,23689,234567,134567,1346,27,27,56,489,568,5689,,3489,348,,48,56,,,,456,4568,,,48,,489,56,,4569,45689,,56,12,,13,,2358,367,1678,168,12,,,,128,248,49,1489,,56,,,,15,345,346,,1346
18   	        625,800	26	 168	  4	Skyscraper                    	Skyscraper: E2, H2, E9, I9 on 1 (H7-1, H8-1, I1-1, I3-1)
*/
private static final String PUZZLE
=".5..4.8...6.5....9...............1..9..231..73.1..7..2..4.9.....376....5.987...2." + NL
+"17,,239,139,,2369,,1367,136,48,,23,,1278,238,2347,1347,,48,17,239,189,12678,23689,234567,134567,1346,27,27,56,489,568,5689,,3489,348,,48,56,,,,456,4568,,,48,,489,56,,4569,45689,,56,12,,13,,2358,367,1678,168,12,,,,128,248,49,1489,,56,,,,15,345,346,,1346";

/** Test Skyscraper */
@Test
public void test() {
	final String testName = testName(this);
	AHinter hinter = new Skyscraper();
	getHints(PUZZLE, hinter);
	AHint hint = find(HINTS, "Skyscraper: E2, H2, E9, I9 on 1");
if(REPRODUCE) {
//KEEP THESE: to maintain this test case.
System.out.format("clueHtml : %s\n", hint.getClueHtml(true));
System.out.format("aquaCells: %s\n", Frmt.csv(hint.getAquaCells(0)));
System.out.format("oranges  : %s\n", hint.getOranges(0));
System.out.format("reds     : %s\n", hint.getReds(0));
System.out.format("bases    : %s\n", diuf.sudoku.utils.Frmt.csv(hint.getBases()));
System.out.format("covers   : %s\n", diuf.sudoku.utils.Frmt.csv(hint.getCovers()));
}
	checkClue(hint, "Look for a Skyscraper on 1");
	assertEquals(grid.getCSVSet("E2, H2, E9, I9"), hint.getAquaCells(0));
	assertEquals(newPots("E2-1, E9-1, H2-1, I9-1"), hint.getOranges(0));
	assertEquals(newPots("H7-1, H8-1, I1-1, I3-1"), hint.getReds(0));
	assertArrayEquals(grid.regions("row 2, row 9"), hint.getBasesArray());
	assertArrayEquals(grid.regions("col H, col I"), hint.getCoversArray());
	assertHintEqualsStoredHintsHtml(this, hint, testName+".html", REPRODUCE);
}

}
