/*
 * Project: Sudoku Explainer
 * Copyright (C) 2006-2007 Nicolas Juillerat
 * Copyright (C) 2013-2020 Keith Corlett
 * Available under the terms of the Lesser General Public License (LGPL)
 */
package diuf.sudoku.solver.hinters.sdp;

import diuf.sudoku.Grid.ARegion;
import diuf.sudoku.Regions;
import static diuf.sudoku.TestHelp.*;
import diuf.sudoku.solver.AHint;
import diuf.sudoku.solver.hinters.AHinter;
import diuf.sudoku.utils.Frmt;
import diuf.sudoku.utils.MyLinkedHashSet;
import org.junit.Test;
import static org.junit.Assert.*;


/**
 * Test the TwoStringKite hinter.
 *
 * @author Keith Corlett 2021-02-08
 */
public class TwoStringKiteTest {

private static final boolean REPRODUCE = REPRODUCE_ALL_TESTS || false;

/*
7....54...2..7..8...32.8..9...5.63...6..2..9...1..7..6...3529...3.749.6...9..1.35
,189,68,16,39,,,12,123,19,,56,49,,34,56,,13,146,145,,,16,,67,57,,289,79,247,,18,,,1247,12478,35,,47,18,,34,1578,,1478,23,58,,49,39,,258,245,,168,178,678,,,,,147,1478,1258,,25,,,,128,,128,24,47,,68,68,,27,,
1#C:\Users\User\Documents\NetBeansProjects\DiufSudoku\top1465.d5.mt
TwoStringKite	Two String Kite: G2, B6 on 5 (G6-5)
*/
private static final String PUZZLE
="7....54...2..7..8...32.8..9...5.63...6..2..9...1..7..6...3529...3.749.6...9..1.35" + NL
+",189,68,16,39,,,12,123,19,,56,49,,34,56,,13,146,145,,,16,,67,57,,289,79,247,,18,,,1247,12478,35,,47,18,,34,1578,,1478,23,58,,49,39,,258,245,,168,178,678,,,,,147,1478,1258,,25,,,,128,,128,24,47,,68,68,,27,,";

/** Test TwoStringKite */
@Test
public void test() {
	final String testName = testName(this);
	AHinter hinter = new TwoStringKite();
	getHints(PUZZLE, hinter);
	AHint hint = find(HINTS, "Two String Kite: G2, B6 on 5");
if(REPRODUCE) {
//KEEP THESE: to maintain this test case.
System.out.format("clueHtml : %s\n", hint.getClueHtml(true));
System.out.format("aquaCells: %s\n", Frmt.csv(hint.getAquaCells(0)));
System.out.format("blues    : %s\n", hint.getBlues(grid, 0));
System.out.format("oranges  : %s\n", hint.getOranges(0));
System.out.format("greens   : %s\n", hint.getGreens(0));
System.out.format("reds     : %s\n", hint.getReds(0));
System.out.format("redPots  : %s\n", hint.redPots);
System.out.format("bases    : %s\n", diuf.sudoku.utils.Frmt.csv(hint.getBases()));
System.out.format("covers   : %s\n", diuf.sudoku.utils.Frmt.csv(hint.getCovers()));
}
	checkClue(hint, "Look for a Two String Kite on 5");
	assertEquals(grid.getCSVSet("G2, B6"), hint.getAquaCells(0));
	assertEquals(newPots("B3-5, C2-5"), hint.getBlues(grid, 0));
	assertEquals(newPots("B6-5, G2-5"), hint.getOranges(0));
	assertEquals(null, hint.getGreens(0));
	assertEquals(newPots("G6-5"), hint.getReds(0));
	assertEquals(newPots("G6-5"), hint.redPots);
	assertArrayEquals(new ARegion[]{grid.boxs[0]}, hint.getBasesArray());
	assertArrayEquals(new ARegion[]{grid.rows[1], grid.cols[1]}, hint.getCoversArray());
	assertHintEqualsStoredHintsHtml(this, hint, testName+".html", REPRODUCE);
}

}
