/*
 * Project: Sudoku Explainer
 * Copyright (C) 2006-2007 Nicolas Juillerat
 * Copyright (C) 2013-2020 Keith Corlett
 * Available under the terms of the Lesser General Public License (LGPL)
 */
package diuf.sudoku.solver.hinters.single;

import diuf.sudoku.Grid.Cell;
import diuf.sudoku.Pots;
import org.junit.Test;
import static org.junit.Assert.*;
import diuf.sudoku.Values;
import diuf.sudoku.solver.AHint;
import diuf.sudoku.utils.MyLinkedHashSet;
import static diuf.sudoku.TestHelp.*;
import diuf.sudoku.solver.hinters.AHinter;

/**
 * Test the HiddenSingle hinter.
 * @author Keith Corlett 2018 Apr
 */
public final class HiddenSingleTest {
	
private static final boolean REPRODUCE = REPRODUCE_ALL_TESTS;

private static final String HDN_SNGL
="7.....4...2..7..8...3..8..9...5..3...6..2..9...1..7..6...3..9...3..4..6...9..1..5";

/** Test of getHints method, of class HiddenSingle. */
@Test
public void testGetHints() {
	String testName = testName(this);
	AHinter hinter = new HiddenSingle();
	getHints(HDN_SNGL, hinter);
	AHint hint = find(HINTS, "Hidden Single: H9=3 in box 9", "HiddenSingleHint");
	Cell h9 = grid.get("H9");
	assertEquals(h9, hint.cell);
	assertEquals(3, hint.value);
	assertEquals(new MyLinkedHashSet<>(h9), hint.getAquaCells(0));
	assertEquals(new Pots(h9, new Values(3)), hint.getGreens(0));
	assertEquals(new Pots(h9, newValues(2,4,7)), hint.getReds(0));
	assertEquals(null, hint.redPots);
	checkClue(hint, "Look for a Hidden Single in <b1>box 9</b1>");
	assertHintEqualsStoredHintsHtml(this, hint, testName+".html", REPRODUCE);
}

}