/*
 * Project: Sudoku Explainer
 * Copyright (C) 2006-2007 Nicolas Juillerat
 * Copyright (C) 2013-2020 Keith Corlett
 * Available under the terms of the Lesser General Public License (LGPL)
 */
package diuf.sudoku.solver.hinters.single;

import diuf.sudoku.Grid.Cell;
import diuf.sudoku.Pots;
import diuf.sudoku.Values;
import diuf.sudoku.solver.AHint;
import diuf.sudoku.utils.MyLinkedHashSet;
import org.junit.Test;
import static org.junit.Assert.*;
import static diuf.sudoku.TestHelp.*;
import diuf.sudoku.solver.hinters.AHinter;


/**
 * Test the NakedSingle hinter.
 * @author Keith Corlett 2018 Apr
 */
public final class NakedSingleTest {

private static final boolean REPRODUCE = REPRODUCE_ALL_TESTS;

private static final String PUZZLE
=".....8.7..13..............4...1.45...9...73....82....65..6..9...2..43.6..6..1....";
//12#C:\Users\User\Documents\NetBeansProjects\DiufSudoku\top1465.d3.mt

/** Test of getHints method, of class NakedSingle. */
@Test
public void testGetHints() {
	String testName = testName(this);
	AHinter hinter = new NakedSingle();
	getHints(PUZZLE, hinter);
	Cell cell = grid.get("F7");
	int value = 2;
	AHint hint = find(HINTS,"Naked Single: F7=2");
	assertEquals(cell, hint.cell);
	assertEquals(value, hint.value);
	assertEquals(new MyLinkedHashSet<>(cell), hint.getAquaCells(0));
	assertEquals(new Pots(cell, new Values(value)), hint.getGreens(0));
	assertEquals(null, hint.getReds(0)); // otherwise it wouldn't be naked
	assertEquals(null, hint.redPots); // otherwise it wouldn't be naked
	assertHintEqualsStoredHintsHtml(this, hint, testName+".html", REPRODUCE);
}

}
