/*
 * Project: Sudoku Explainer
 * Copyright (C) 2006-2007 Nicolas Juillerat
 * Copyright (C) 2013-2020 Keith Corlett
 * Available under the terms of the Lesser General Public License (LGPL)
 */
package diuf.sudoku.solver.hinters.wing;

import diuf.sudoku.Tech;
import org.junit.Test;
import static org.junit.Assert.*;
import diuf.sudoku.solver.AHint;
import diuf.sudoku.utils.MyLinkedHashSet;
import static diuf.sudoku.TestHelp.*;
import diuf.sudoku.solver.hinters.AHinter;
import diuf.sudoku.utils.Frmt;


/**
 * Test the hinter.
 * @author Keith Corlett 2021 Jan
 */
public final class STUVWXYZWingTest {

private static final boolean REPRODUCE = REPRODUCE_ALL_TESTS || false;

/*
42#C:\Users\User\Documents\NetBeansProjects\DiufSudoku\top1465.d5.mt
2..7.91......5..9...1.64..3......8......4..6...9..6..27..5..6...5.67..3...64.3..1
,3468,3458,,38,,,48,56,3468,3468,3478,1238,,128,24,,67,59,79,,28,,,257,2578,,346,12346,23457,1239,12,57,,14,4579,138,1238,23578,12389,,57,39,,579,3458,13478,,138,138,,3457,1457,,,123489,2348,,189,128,,24,489,1489,,248,,,128,249,,489,89,289,,,289,,57,57,
20   	        818,900	27	 170	  3	STUVWXYZ-Wing                 	STUVWXYZ-Wing: A4 B4 C4 E4 F4 H4 I4 and G5 on 3 (A5-3, B5-3, C5-3)
*/
private static final String STUVWXYZ_WING
="2..7.91......5..9...1.64..3......8......4..6...9..6..27..5..6...5.67..3...64.3..1" + NL
+",3468,3458,,38,,,48,56,3468,3468,3478,1238,,128,24,,67,59,79,,28,,,257,2578,,346,12346,23457,1239,12,57,,14,4579,138,1238,23578,12389,,57,39,,579,3458,13478,,138,138,,3457,1457,,,123489,2348,,189,128,,24,489,1489,,248,,,128,249,,489,89,289,,,289,,57,57,";

/** Test STUVWXYZWing */
@Test
public void test() {
	final String testName = testName(this);
	AHinter hinter = new BigWing(Tech.STUVWXYZ_Wing);
	getHints(STUVWXYZ_WING, hinter);
	AHint hint = find(HINTS, "STUVWXYZ-Wing: A4 B4 C4 E4 F4 H4 I4 and G5 on 3");
if ( REPRODUCE ) {
// KEEP THESE to maintain this test case.
System.out.format("clueHtml : %s\n", hint.getClueHtml(true));
System.out.format("aquaCells: %s\n", Frmt.csv(hint.getAquaCells(0)));
System.out.format("pinkCells: %s\n", Frmt.csv(hint.getPinkCells(0)));
System.out.format("greens   : %s\n", hint.getGreens(0));
System.out.format("oranges  : %s\n", hint.getOranges(0));
System.out.format("reds     : %s\n", hint.getReds(0));
System.out.format("redPots  : %s\n", hint.redPots);
System.out.format("bases    : %s\n", diuf.sudoku.utils.Frmt.csv(hint.getBases()));
System.out.format("covers   : %s\n", diuf.sudoku.utils.Frmt.csv(hint.getCovers()));
}
	checkClue(hint, "Look for a STUVWXYZ-Wing on 1, 2, 4, 5, 6, 7 and <b>3</b>");
	assertEquals(grid.getCSVSet("A4, B4, C4, E4, F4, H4, I4"), hint.getAquaCells(0));
	assertEquals(grid.getCSVSet("G5"), hint.getPinkCells(0));
	assertEquals(newPots("A4-3, B4-3, C4-3, G5-3"), hint.getGreens(0));
	assertEquals(null, hint.getOranges(0));
	assertEquals(newPots("E5-1"), hint.getReds(0));
	assertEquals(newPots("E5-1"), hint.redPots);
	assertEquals(null, hint.getBases());
	assertEquals(null, hint.getCovers());
	assertHintEqualsStoredHintsHtml(this, hint, testName+".html", REPRODUCE);
}

}