/*
 * Project: Sudoku Explainer
 * Copyright (C) 2006-2007 Nicolas Juillerat
 * Copyright (C) 2013-2020 Keith Corlett
 * Available under the terms of the Lesser General Public License (LGPL)
 */
package diuf.sudoku.solver.hinters.wing;

import diuf.sudoku.Tech;
import org.junit.Test;
import static org.junit.Assert.*;
import diuf.sudoku.solver.AHint;
import diuf.sudoku.utils.MyLinkedHashSet;
import static diuf.sudoku.TestHelp.*;
import diuf.sudoku.solver.hinters.AHinter;
import diuf.sudoku.utils.Frmt;

/**
 * Test the hinter.
 * @author Keith Corlett 2021 Jan
 */
public final class TUVWXYZWingTest {

private static final boolean REPRODUCE = REPRODUCE_ALL_TESTS || false;

/*
131#C:\Users\User\Documents\NetBeansProjects\DiufSudoku\top1465.d5.mt
.8..7..9.7....8..4.42..187...47....88.72.5..95...8.7...786..13.2..8......6..3..8.
136,,356,45,,2346,2356,,12356,,1359,1369,359,269,,2356,1256,,369,,,359,569,,,,356,1369,1239,,,169,369,2356,1256,,,13,,,146,,346,146,,,239,369,1349,,3469,,246,123,49,,,,2459,249,,,25,,359,1359,,1459,479,4569,456,567,149,,159,1459,,2479,2459,,27
18   	      7,970,800	32	 162	  1	TUVWXYZ-Wing                  	TUVWXYZ-Wing 1420: B8 E8 F8 G8 H8 I8 B5 on value 1 (E5-1)
*/
private static final String TUVWXYZ_WING
=".8..7..9.7....8..4.42..187...47....88.72.5..95...8.7...786..13.2..8......6..3..8." + NL
+"136,,356,45,,2346,2356,,12356,,1359,1369,359,269,,2356,1256,,369,,,359,569,,,,356,1369,1239,,,169,369,2356,1256,,,13,,,146,,346,146,,,239,369,1349,,3469,,246,123,49,,,,2459,249,,,25,,359,1359,,1459,479,4569,456,567,149,,159,1459,,2479,2459,,27";

/** Test TUVWXYZWing */
@Test
public void test() {
	final String testName = testName(this);
	AHinter hinter = new BigWing(Tech.TUVWXYZ_Wing);
	getHints(TUVWXYZ_WING, hinter);
	AHint hint = find(HINTS, "TUVWXYZ-Wing: B8 E8 F8 G8 H8 I8 and B5 on 1");
if ( REPRODUCE ) {
// KEEP THESE to maintain this test case.
System.out.format("SimpleName : %s\n", hint.getClass().getSimpleName());
System.out.format("cell       : %s\n", hint.cell);
System.out.format("value      : %s\n", hint.value);
System.out.format("ClueHtml   : %s\n", hint.getClueHtml(true));
System.out.format("AquaCells  : %s\n", Frmt.csv(hint.getAquaCells(0)));
System.out.format("Oranges    : %s\n", hint.getOranges(0));
System.out.format("Greens     : %s\n", hint.getGreens(0));
System.out.format("Reds       : %s\n", hint.getReds(0));
System.out.format("RedPots    : %s\n", hint.redPots);
System.out.format("bases      : %s\n", diuf.sudoku.utils.Frmt.csv(hint.getBases()));
System.out.format("covers     : %s\n", diuf.sudoku.utils.Frmt.csv(hint.getCovers()));
}
	checkClue(hint, "Look for a TUVWXYZ-Wing on 4, 5, 6, 7, 9 and <b>1</b>");
	assertEquals(grid.getCSVSet("I8, B8, E8, F8, G8, H8"), hint.getAquaCells(0));
	assertEquals(null, hint.getOranges(0));
	assertEquals(newPots("B5-1, E8-1"), hint.getGreens(0));
	assertEquals(newPots("E5-1"), hint.getReds(0));
	assertEquals(newPots("E5-1"), hint.redPots);
	assertEquals(null, hint.getBases());
	assertEquals(null, hint.getCovers());
	assertHintEqualsStoredHintsHtml(this, hint, testName+".html", REPRODUCE);
}

}