/*
 * Project: Sudoku Explainer
 * Copyright (C) 2006-2007 Nicolas Juillerat
 * Copyright (C) 2013-2020 Keith Corlett
 * Available under the terms of the Lesser General Public License (LGPL)
 */
package diuf.sudoku.solver.hinters.wing;

import diuf.sudoku.Tech;
import org.junit.Test;
import static org.junit.Assert.*;
import diuf.sudoku.solver.AHint;
import diuf.sudoku.utils.MyLinkedHashSet;
import static diuf.sudoku.TestHelp.*;
import diuf.sudoku.solver.hinters.AHinter;
import diuf.sudoku.utils.Frmt;

/**
 * Test the hinter.
 * @author Keith Corlett 2021 Jan
 */
public final class UVWXYZWingTest {
	
private static final boolean REPRODUCE = REPRODUCE_ALL_TESTS || false;

/*
1#C:\Users\User\Documents\NetBeansProjects\DiufSudoku\top1465.d5.mt
7.....4...2..7..8...3..8..9...5.63...6..2..9...1..7..6...3.29...3..4..6...9..1.35
,189,68,1269,1369,59,,125,123,19,,56,149,,34,56,,13,146,145,,126,156,,1267,1257,,289,789,247,,189,,,1247,12478,35,,47,18,,34,1578,,1478,23,589,,489,39,,258,245,,1568,1578,678,,568,,,147,1478,1258,,257,79,,59,1278,,1278,2468,478,,678,68,,278,,
38   	      2,924,000	27	 160	  2	UVWXYZ-Wing                   	UVWXYZ-Wing: A4 B4 C4 C5 A6 E6 on value 9 (B6-9, E4-9)
*/
private static final String UVWXYZ_WING
="7.....4...2..7..8...3..8..9...5.63...6..2..9...1..7..6...3.29...3..4..6...9..1.35" + NL
+",189,68,1269,1369,59,,125,123,19,,56,149,,34,56,,13,146,145,,126,156,,1267,1257,,289,789,247,,189,,,1247,12478,35,,47,18,,34,1578,,1478,23,589,,489,39,,258,245,,1568,1578,678,,568,,,147,1478,1258,,257,79,,59,1278,,1278,2468,478,,678,68,,278,,";

/** Test UVWXYZWing */
@Test
public void test() {
	final String testName = testName(this);
	AHinter hinter = new BigWing(Tech.UVWXYZ_Wing);
	getHints(UVWXYZ_WING, hinter);
	AHint hint = find(HINTS, "UVWXYZ-Wing: A4 B4 C4 C5 A6 and E6 on 9");
if ( REPRODUCE ) {
// KEEP THESE to maintain this test case.
System.out.format("SimpleName : %s\n", hint.getClass().getSimpleName());
System.out.format("cell       : %s\n", hint.cell);
System.out.format("value      : %s\n", hint.value);
System.out.format("ClueHtml   : %s\n", hint.getClueHtml(true));
System.out.format("AquaCells  : %s\n", Frmt.csv(hint.getAquaCells(0)));
System.out.format("Oranges    : %s\n", hint.getOranges(0));
System.out.format("Greens     : %s\n", hint.getGreens(0));
System.out.format("Reds       : %s\n", hint.getReds(0));
System.out.format("RedPots    : %s\n", hint.redPots);
System.out.format("bases      : %s\n", diuf.sudoku.utils.Frmt.csv(hint.getBases()));
System.out.format("covers     : %s\n", diuf.sudoku.utils.Frmt.csv(hint.getCovers()));
}
	checkClue(hint, "Look for a UVWXYZ-Wing on 2, 4, 7, 8 and <b>9</b>");
	assertEquals(grid.getCSVSet("A4, A6, B4, C4, C5"), hint.getAquaCells(0));
	assertEquals(null, hint.getOranges(0));
	assertEquals(newPots("A4-9, B4-9, E6-9"), hint.getGreens(0));
	assertEquals(newPots("B6-9, E4-9"), hint.getReds(0));
	assertEquals(newPots("B6-9, E4-9"), hint.redPots);
	assertEquals(null, hint.getBases());
	assertEquals(null, hint.getCovers());
	assertHintEqualsStoredHintsHtml(this, hint, testName+".html", REPRODUCE);
}

}