/*
 * Project: Sudoku Explainer
 * Copyright (C) 2006-2007 Nicolas Juillerat
 * Copyright (C) 2013-2020 Keith Corlett
 * Available under the terms of the Lesser General Public License (LGPL)
 */
package diuf.sudoku.solver.hinters.wing;

import diuf.sudoku.Tech;
import org.junit.Test;
import static org.junit.Assert.*;
import diuf.sudoku.solver.AHint;
import diuf.sudoku.utils.MyLinkedHashSet;
import static diuf.sudoku.TestHelp.*;
import diuf.sudoku.solver.hinters.AHinter;
import diuf.sudoku.utils.Frmt;

/**
 * Test the hint.
 * @author Keith Corlett 2021 Jan
 */
public final class VWXYZWingTest {

private static final boolean REPRODUCE = REPRODUCE_ALL_TESTS || false;

/*
1#C:\Users\User\Documents\NetBeansProjects\DiufSudoku\top1465.d5.mt
....83....915.4.3.36.....4..1......623.1.94....5........3..6.91....4.2.3..73....4
457,457,24,26,,,19,1267,279,78,,,,267,,678,,278,,,28,79,19,127,578,,2578,4789,,489,2478,2357,2578,39,2578,,,,68,,567,,,578,578,46789,478,,24678,2367,278,139,1278,2789,458,2458,,278,257,,578,,,169,58,69,789,,1578,,5678,,169,258,,,19,1258,568,568,
18   	        739,400	28	 164	  1	VWXYZ-Wing                    	VWXYZ-Wing1: B1 B7 B8 B9 A2 on value 8 (A7-8)
*/
private static final String VWXYZ_WING
="....83....915.4.3.36.....4..1......623.1.94....5........3..6.91....4.2.3..73....4" + NL
+"457,457,24,26,,,19,1267,279,78,,,,267,,678,,278,,,28,79,19,127,578,,2578,4789,,489,2478,2357,2578,39,2578,,,,68,,567,,,578,578,46789,478,,24678,2367,278,139,1278,2789,458,2458,,278,257,,578,,,169,58,69,789,,1578,,5678,,169,258,,,19,1258,568,568,";

/** Test VWXYZWing */
@Test
public void test() {
	final String testName = testName(this);
	AHinter hinter = new BigWing(Tech.VWXYZ_Wing);
	getHints(VWXYZ_WING, hinter);
	AHint hint = find(HINTS, "VWXYZ-Wing: B1 B7 B8 B9 and A2 on 8");
if ( REPRODUCE ) {
// KEEP THESE to maintain this testcase.
System.out.format("SimpleName : %s\n", hint.getClass().getSimpleName());
System.out.format("cell       : %s\n", hint.cell);
System.out.format("value      : %s\n", hint.value);
System.out.format("ClueHtml   : %s\n", hint.getClueHtml(true));
System.out.format("AquaCells  : %s\n", Frmt.csv(hint.getAquaCells(0)));
System.out.format("Oranges    : %s\n", hint.getOranges(0));
System.out.format("Greens     : %s\n", hint.getGreens(0));
System.out.format("Reds       : %s\n", hint.getReds(0));
System.out.format("RedPots    : %s\n", hint.redPots);
System.out.format("bases      : %s\n", diuf.sudoku.utils.Frmt.csv(hint.getBases()));
System.out.format("covers     : %s\n", diuf.sudoku.utils.Frmt.csv(hint.getCovers()));
}
	checkClue(hint, "Look for a VWXYZ-Wing on 2, 4, 5 and <b>8</b>");
	assertEquals(grid.getCSVSet("B1, B7, B8, B9"), hint.getAquaCells(0));
	assertEquals(null, hint.getOranges(0));
	assertEquals(newPots("A2-8, B7-8, B8-8, B9-8"), hint.getGreens(0));
	assertEquals(newPots("A7-8"), hint.getReds(0));
	assertEquals(newPots("A7-8"), hint.redPots);
	assertEquals(null, hint.getBases());
	assertEquals(null, hint.getCovers());
	assertHintEqualsStoredHintsHtml(this, hint, testName+".html", REPRODUCE);
}

}
