/*
 * Project: Sudoku Explainer
 * Copyright (C) 2006-2007 Nicolas Juillerat
 * Copyright (C) 2013-2020 Keith Corlett
 * Available under the terms of the Lesser General Public License (LGPL)
 */
package diuf.sudoku.solver.hinters.wing;

import org.junit.Test;
import static org.junit.Assert.*;
import diuf.sudoku.solver.AHint;
import diuf.sudoku.utils.MyLinkedHashSet;
import static diuf.sudoku.TestHelp.*;
import diuf.sudoku.solver.hinters.AHinter;

/**
 * Test the WWing hinter.
 * @author Keith Corlett 2020 Apr
 */
public final class WWingTest {
	
private static final boolean REPRODUCE = REPRODUCE_ALL_TESTS;

private static final String W_WING
=".7..28..3386175249..234.7..8.7...3.22.9.3.1..513.82.......6382762.8.793.73829..6." + NL
+"149,,145,69,,,56,15,,,,,,,,,,,19,59,,,,69,,158,568,,46,,4569,15,1469,,59,,,46,,57,,46,,578,58,,,,79,,,46,79,46,149,59,145,45,,,,,,,,45,,15,,,,14,,,,,,14,45,,145";
//C:/Users/User/Documents/SodukuPuzzles/HoDoKu/W-Wing_001.txt
//W_Wing	W-Wing: B3, H4 on 5 and 9 (H3-5)

/** Test W-Wing */
@Test
public void testWWing() {
	final String testName = testName(this);
	AHinter hinter = new WWing();
	getHints(W_WING, hinter);
	AHint hint = find(HINTS, "W-Wing: B3, H4, F3, F4 on 5 and 9");
//// Keep these, it's how I create a test case. I'm only resting for repeated.
//System.out.format("SimpleName : %s\n", hint.getClass().getSimpleName());
//System.out.format("cell       : %s\n", hint.cell);
//System.out.format("value      : %s\n", hint.value);
//System.out.format("ClueHtml   : %s\n", hint.getClueHtml(true));
//System.out.format("AquaCells  : %s\n", Frmt.csv(hint.getAquaCells(0)));
//System.out.format("Oranges    : %s\n", hint.getOranges(0));
//System.out.format("Greens     : %s\n", hint.getGreens(0));
//System.out.format("Reds       : %s\n", hint.getReds(0));
//System.out.format("RedPots    : %s\n", hint.redPots);
//System.out.format("bases      : %s\n", diuf.sudoku.utils.Frmt.csv(hint.getBases()));
//System.out.format("covers     : %s\n", diuf.sudoku.utils.Frmt.csv(hint.getCovers()));
	checkClue(hint, "Look for a W-Wing on 5 and 9");
	assertEquals(grid.getCSVSet("F3, F4, B3, H4"), hint.getAquaCells(0));
	assertEquals(null, hint.getOranges(0));
	assertEquals(newPots("B3-9, F3-9, F4-9, H4-9"), hint.getGreens(0));
	assertEquals(newPots("H3-5"), hint.getReds(0));
	assertEquals(newPots("H3-5"), hint.redPots);
	assertEquals(null, hint.getBases());
	assertEquals(null, hint.getCovers());
	assertHintEqualsStoredHintsHtml(this, hint, testName+"_1.html", REPRODUCE);
}

}