/*
 * Project: Sudoku Explainer
 * Copyright (C) 2006-2007 Nicolas Juillerat
 * Copyright (C) 2013-2020 Keith Corlett
 * Available under the terms of the Lesser General Public License (LGPL)
 */
package diuf.sudoku.solver.hinters.wing;

import diuf.sudoku.Tech;
import org.junit.Test;
import static org.junit.Assert.*;
import diuf.sudoku.solver.AHint;
import diuf.sudoku.utils.MyLinkedHashSet;
import static diuf.sudoku.TestHelp.*;
import diuf.sudoku.solver.hinters.AHinter;
import diuf.sudoku.utils.Frmt;

/**
 * Test the hinter.
 * @author Keith Corlett 2021 Jan
 */
public final class WXYZWingTest {

private static final boolean REPRODUCE = REPRODUCE_ALL_TESTS || false;

/*
1#C:\Users\User\Documents\NetBeansProjects\DiufSudoku\top1465.d5.mt
7....54...2..7..8...32.8..9...5.63...6..2..9...1..7..6...3529...3.749.6...9..1.35
,189,68,16,39,,,12,123,19,,56,49,,34,56,,13,146,145,,,16,,67,57,,289,79,247,,18,,,1247,12478,35,,47,18,,34,1578,,478,23,58,,49,39,,28,245,,168,178,678,,,,,147,478,1258,,25,,,,128,,128,24,47,,68,68,,27,,
61   	      1,331,800	32	 121	  1	WXYZ-Wing                     	WXYZ-Wing 126: A5 A6 B6 G6 on value 2 (H6-2)
*/
private static final String WXYZ_WING
="7....54...2..7..8...32.8..9...5.63...6..2..9...1..7..6...3529...3.749.6...9..1.35" + NL
+",189,68,16,39,,,12,123,19,,56,49,,34,56,,13,146,145,,,16,,67,57,,289,79,247,,18,,,1247,12478,35,,47,18,,34,1578,,478,23,58,,49,39,,28,245,,168,178,678,,,,,147,478,1258,,25,,,,128,,128,24,47,,68,68,,27,,";

/** Test WXYZWing */
@Test
public void test() {
	final String testName = testName(this);
	AHinter hinter = new BigWing(Tech.WXYZ_Wing);
	getHints(WXYZ_WING, hinter);
	AHint hint = find(HINTS, "WXYZ-Wing: A6 B6 G6 and A5 on 3 and 5");
if ( REPRODUCE ) {
// KEEP THESE to maintain this test case.
System.out.format("SimpleName : %s\n", hint.getClass().getSimpleName());
System.out.format("cell       : %s\n", hint.cell);
System.out.format("value      : %s\n", hint.value);
System.out.format("ClueHtml   : %s\n", hint.getClueHtml(true));
System.out.format("AquaCells  : %s\n", Frmt.csv(hint.getAquaCells(0)));
System.out.format("Oranges    : %s\n", hint.getOranges(0));
System.out.format("Greens     : %s\n", hint.getGreens(0));
System.out.format("Reds       : %s\n", hint.getReds(0));
System.out.format("RedPots    : %s\n", hint.redPots);
System.out.format("bases      : %s\n", diuf.sudoku.utils.Frmt.csv(hint.getBases()));
System.out.format("covers     : %s\n", diuf.sudoku.utils.Frmt.csv(hint.getCovers()));
}
	checkClue(hint, "Look for a WXYZ-Wing on 2, 8 and <b>5</b>");
	assertEquals(grid.getCSVSet("A6, B6, G6"), hint.getAquaCells(0));
	assertEquals(null, hint.getOranges(0));
	assertEquals(newPots("A5-35, A6-23, B6-58, G6-28"), hint.getGreens(0));
	assertEquals(newPots("H6-2"), hint.getReds(0));
	assertEquals(newPots("H6-2"), hint.redPots);
	assertEquals(null, hint.getBases());
	assertEquals(null, hint.getCovers());
	assertHintEqualsStoredHintsHtml(this, hint, testName+".html", REPRODUCE);
}

}