/*
 * Project: Sudoku Explainer
 * Copyright (C) 2006-2007 Nicolas Juillerat
 * Copyright (C) 2013-2020 Keith Corlett
 * Available under the terms of the Lesser General Public License (LGPL)
 */
package diuf.sudoku.utils;

import static org.junit.Assert.*;
import static diuf.sudoku.TestHelp.*;
import java.util.LinkedHashSet;
import org.junit.Test;

/**
 * Tests the static helper methods in the utils.Frmt (formatting) class.
 * Currently only the csvToSetOfStrings method is tested, and there's ancillary-testing
 of the {@code Frmt.csvs(Collection<String>)} method, which I already know
 * works.
 * @author Keith Corlett 2019 OCT
 */
public class FrmtTest {
@Test
public void test000_fromCsv() {
	final String testName = testName(this); // retain testName even if not used
	LinkedHashSet<String> set;
	
	// check that empty sep throws an IllegalArgumentException
	try {
		set = Frmt.csvToSetOfStrings("a, normal, list", "");
		assert false : "You can't get here!";
	} catch (IllegalArgumentException ex) {
		// PASS: I care not what the message is
	}
	
	// check that null sep throws an IllegalArgumentException
	try {
		set = Frmt.csvToSetOfStrings("a, normal, list", null);
		assert false : "You can't get here!";
	} catch (IllegalArgumentException ex) {
		// PASS: I care not what the message is
	}

	// check that a null line produces an empty set, not an error.
	set = Frmt.csvToSetOfStrings(null, ", ");
	assertEquals(0, set.size());
	assertEquals("", Frmt.csvs(set));
	
	// check that an empty line produces an empty set, not an error.
	set = Frmt.csvToSetOfStrings("", ", ");
	assertEquals(0, set.size());
	assertEquals("", Frmt.csvs(set));

	// check that a line with no seperator produces a set of 1 element.
	set = Frmt.csvToSetOfStrings("0", ", ");
	assertEquals(1, set.size());
	assertEquals("0", Frmt.csvs(set));

	// check "normal operation" seperator length == 2
	set = Frmt.csvToSetOfStrings("0, 1, 2", ", ");
	assertEquals(3, set.size());
	assertEquals("0, 1, 2", Frmt.csvs(set));
	
	// check "the more unusual option" seperator length == 1
	set = Frmt.csvToSetOfStrings("0,1,2", ",");
	assertEquals(3, set.size());
	assertEquals("0, 1, 2", Frmt.csvs(set));

	// check "the highly unusual option" seperator length > 2
	set = Frmt.csvToSetOfStrings("0.. 1.. 2", ".. ");
	assertEquals(3, set.size());
	assertEquals("0, 1, 2", Frmt.csvs(set));
	
	// check that when my caller buggers it up I don't. A common mistake is
	// passing "," when you meant ", " which results in a SLIGHTLY screwie
	// Set of elements with leading spaces; ie it doesn't "blow-up".
	set = Frmt.csvToSetOfStrings("0, 1, 2", ",");
	assertEquals(3, set.size());
	assertEquals("0,  1,  2", Frmt.csvs(set));

	// now here's a weird one for ya! can you knut-this out yourself?
	set = Frmt.csvToSetOfStrings("0,1,2", ", ");
	assertEquals(1, set.size());
	assertEquals("0,1,2", Frmt.csvs(set));
}
}
